@@ -13,6 +13,8 @@ import static com.google.common.math.LongMath.checkedAdd;
 import static com.google.common.math.LongMath.checkedMultiply;
 import static com.google.common.math.LongMath.checkedSubtract;
 
+import com.google.common.annotations.GwtIncompatible;
+import com.google.common.base.Strings;
 import com.google.errorprone.annotations.CanIgnoreReturnValue;
 import com.google.errorprone.annotations.CompileTimeConstant;
 import com.google.j2objc.annotations.J2ObjCIncompatible;
@@ -62,6 +64,8 @@ public final class Timestamps {
    */
   public static final Timestamp EPOCH = Timestamp.newBuilder().setSeconds(0).setNanos(0).build();
 
+  @GwtIncompatible("Date formatting is not supported in non JVM java.time")
+  @J2ObjCIncompatible
   private static final ThreadLocal<SimpleDateFormat> timestampFormat =
       new ThreadLocal<SimpleDateFormat>() {
         @Override
@@ -70,6 +74,8 @@ public final class Timestamps {
         }
       };
 
+  @GwtIncompatible("Date formatting is not supported in non JVM java.time")
+  @J2ObjCIncompatible
   private static SimpleDateFormat createTimestampFormat() {
     SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss", Locale.ENGLISH);
     GregorianCalendar calendar = new GregorianCalendar(TimeZone.getTimeZone("UTC"));
@@ -163,7 +169,7 @@ public final class Timestamps {
     int nanos = timestamp.getNanos();
     if (!isValid(seconds, nanos)) {
       throw new IllegalArgumentException(
-          String.format(
+          Strings.lenientFormat(
               "Timestamp is not valid. See proto definition for valid values. "
                   + "Seconds (%s) must be in range [-62,135,596,800, +253,402,300,799]. "
                   + "Nanos (%s) must be in range [0, +999,999,999].",
@@ -193,6 +199,8 @@ public final class Timestamps {
    * @return The string representation of the given timestamp.
    * @throws IllegalArgumentException if the given timestamp is not in the valid range.
    */
+  @GwtIncompatible("Depends on String.format which is not supported in Xplat.")
+  @J2ObjCIncompatible
   public static String toString(Timestamp timestamp) {
     checkValid(timestamp);
 
@@ -222,6 +230,8 @@ public final class Timestamps {
    * @return a Timestamp parsed from the string
    * @throws ParseException if parsing fails
    */
+  @GwtIncompatible("ParseException is not supported in Xplat")
+  @J2ObjCIncompatible
   public static Timestamp parse(String value) throws ParseException {
     int dayOffset = value.indexOf('T');
     if (dayOffset == -1) {
@@ -287,6 +297,8 @@ public final class Timestamps {
    * @return a {@link Timestamp} parsed from the string
    * @throws IllegalArgumentException if parsing fails
    */
+  @GwtIncompatible("Depends on String.format which is not supported in Xplat.")
+  @J2ObjCIncompatible
   public static Timestamp parseUnchecked(@CompileTimeConstant String value) {
     try {
       return parse(value);
@@ -299,12 +311,23 @@ public final class Timestamps {
 
   // the following 3 constants contain references to java.time.Instant methods (if that class is
   // available at runtime); otherwise, they are null.
-  @Nullable private static final Method INSTANT_NOW = instantMethod("now");
+  @GwtIncompatible("Uses reflection to access methods of java.time.Instant")
+  @J2ObjCIncompatible
+  @Nullable
+  private static final Method INSTANT_NOW = instantMethod("now");
 
-  @Nullable private static final Method INSTANT_GET_EPOCH_SECOND = instantMethod("getEpochSecond");
+  @GwtIncompatible("Uses reflection to access methods of java.time.Instant")
+  @J2ObjCIncompatible
+  @Nullable
+  private static final Method INSTANT_GET_EPOCH_SECOND = instantMethod("getEpochSecond");
 
-  @Nullable private static final Method INSTANT_GET_NANO = instantMethod("getNano");
+  @GwtIncompatible("Uses reflection to access methods of java.time.Instant")
+  @J2ObjCIncompatible
+  @Nullable
+  private static final Method INSTANT_GET_NANO = instantMethod("getNano");
 
+  @GwtIncompatible("Uses reflection to access methods of java.time.Instant")
+  @J2ObjCIncompatible
   @Nullable
   private static Method instantMethod(String methodName) {
     try {
@@ -321,6 +344,9 @@ public final class Timestamps {
    * you're unable to mock the current time. Instead, you may want to consider injecting a clock
    * instance to read the current time.
    */
+  @GwtIncompatible("Uses reflection to access methods of java.time.Instant")
+  @J2ObjCIncompatible
+  @SuppressWarnings("nullness")
   public static Timestamp now() {
     if (INSTANT_NOW != null) {
       try {
@@ -469,7 +495,7 @@ public final class Timestamps {
     // when normalized.
     if (!isValidSeconds(seconds)) {
       throw new IllegalArgumentException(
-          String.format(
+          Strings.lenientFormat(
               "Timestamp is not valid. Input seconds is too large. "
                   + "Seconds (%s) must be in range [-62,135,596,800, +253,402,300,799]. ",
               seconds));
@@ -488,6 +514,8 @@ public final class Timestamps {
     return checkValid(timestamp);
   }
 
+  @GwtIncompatible("Depends on String.format which is not supported in Xplat.")
+  @J2ObjCIncompatible
   private static long parseTimezoneOffset(String value) throws ParseException {
     int pos = value.indexOf(':');
     if (pos == -1) {
@@ -504,6 +532,8 @@ public final class Timestamps {
     }
   }
 
+  @GwtIncompatible("Depends on String.format which is not supported in Xplat.")
+  @J2ObjCIncompatible
   static int parseNanos(String value) throws ParseException {
     int result = 0;
     for (int i = 0; i < 9; ++i) {
@@ -519,6 +549,8 @@ public final class Timestamps {
   }
 
   /** Format the nano part of a timestamp or a duration. */
+  @GwtIncompatible("Depends on String.format which is not supported in Xplat.")
+  @J2ObjCIncompatible
   static String formatNanos(int nanos) {
     // Determine whether to use 3, 6, or 9 digits for the nano part.
     if (nanos % NANOS_PER_MILLISECOND == 0) {
