@@ -16,6 +16,16 @@
 #import "GPBUtilities_PackagePrivate.h"
 #import "GPBWireFormat.h"
 
+@interface GPBFileDescriptor ()
+- (instancetype)initWithPackage:(NSString *)package objcPrefix:(NSString *)objcPrefix;
+- (instancetype)initWithPackage:(NSString *)package;
+@end
+
+@interface GPBOneofDescriptor ()
+// name must be long lived.
+- (instancetype)initWithName:(const char *)name fields:(NSArray *)fields;
+@end
+
 @interface GPBDescriptor ()
 - (instancetype)initWithClass:(Class)messageClass
                   messageName:(NSString *)messageName
@@ -51,7 +61,6 @@
 // The addresses of these variables are used as keys for objc_getAssociatedObject.
 static const char kTextFormatExtraValueKey = 0;
 static const char kParentClassValueKey = 0;
-static const char kClassNameSuffixKey = 0;
 static const char kFileDescriptorCacheKey = 0;
 
 static NSArray *NewFieldsArrayForHasIndex(int hasIndex, NSArray *allMessageFields)
@@ -91,11 +100,11 @@ static NSArray *NewFieldsArrayForHasIndex(int hasIndex, NSArray *allMessageField
   // Compute the unknown flags by this version of the runtime and then check the passed in flags
   // (from the generated code) to detect when sources from a newer version are being used with an
   // older runtime.
-  GPBDescriptorInitializationFlags unknownFlags =
-      ~(GPBDescriptorInitializationFlag_FieldsWithDefault |
-        GPBDescriptorInitializationFlag_WireFormat | GPBDescriptorInitializationFlag_UsesClassRefs |
-        GPBDescriptorInitializationFlag_Proto3OptionalKnown |
-        GPBDescriptorInitializationFlag_ClosedEnumSupportKnown);
+  GPBDescriptorInitializationFlags unknownFlags = (GPBDescriptorInitializationFlags)(~(
+      GPBDescriptorInitializationFlag_FieldsWithDefault |
+      GPBDescriptorInitializationFlag_WireFormat | GPBDescriptorInitializationFlag_UsesClassRefs |
+      GPBDescriptorInitializationFlag_Proto3OptionalKnown |
+      GPBDescriptorInitializationFlag_ClosedEnumSupportKnown));
   if ((flags & unknownFlags) != 0) {
     GPBRuntimeMatchFailure();
   }
@@ -108,10 +117,8 @@ static NSArray *NewFieldsArrayForHasIndex(int hasIndex, NSArray *allMessageField
   NSAssert((flags & GPBDescriptorInitializationFlag_ClosedEnumSupportKnown) != 0,
            @"Internal error: close enum should be known");
 
-  // `messageName` and `fileDescription` should both be set or both be unset depending on if this is
-  // being called from current code generation or legacy code generation.
-  NSAssert((messageName == nil) == (fileDescription == NULL),
-           @"name and fileDescription should always be provided together");
+  NSAssert((messageName != nil), @"Internal error: missing messageName");
+  NSAssert((fileDescription != NULL), @"Internal error: missing fileDescription");
 #endif
 
   NSMutableArray *fields =
@@ -137,10 +144,10 @@ static NSArray *NewFieldsArrayForHasIndex(int hasIndex, NSArray *allMessageField
   }
   // No real value in checking all the fields individually, just check the combined flags at the
   // end.
-  GPBFieldFlags unknownFieldFlags =
-      ~(GPBFieldRequired | GPBFieldRepeated | GPBFieldPacked | GPBFieldOptional |
-        GPBFieldHasDefaultValue | GPBFieldClearHasIvarOnZero | GPBFieldTextFormatNameCustom |
-        GPBFieldHasEnumDescriptor | GPBFieldMapKeyMask | GPBFieldClosedEnum);
+  GPBFieldFlags unknownFieldFlags = (GPBFieldFlags)(~(
+      GPBFieldRequired | GPBFieldRepeated | GPBFieldPacked | GPBFieldOptional |
+      GPBFieldHasDefaultValue | GPBFieldClearHasIvarOnZero | GPBFieldTextFormatNameCustom |
+      GPBFieldHasEnumDescriptor | GPBFieldMapKeyMask | GPBFieldClosedEnum));
   if ((mergedFieldFlags & unknownFieldFlags) != 0) {
     GPBRuntimeMatchFailure();
   }
@@ -156,114 +163,6 @@ static NSArray *NewFieldsArrayForHasIndex(int hasIndex, NSArray *allMessageField
   return descriptor;
 }
 
-+ (instancetype)allocDescriptorForClass:(Class)messageClass
-                                   file:(GPBFileDescriptor *)file
-                                 fields:(void *)fieldDescriptions
-                             fieldCount:(uint32_t)fieldCount
-                            storageSize:(uint32_t)storageSize
-                                  flags:(GPBDescriptorInitializationFlags)flags {
-  GPBInternalCompileAssert(GOOGLE_PROTOBUF_OBJC_MIN_SUPPORTED_VERSION <= 30006,
-                           time_to_remove_this_old_version_shim);
-
-  BOOL fixClassRefs = (flags & GPBDescriptorInitializationFlag_UsesClassRefs) == 0;
-  GPBInternalCompileAssert(GOOGLE_PROTOBUF_OBJC_MIN_SUPPORTED_VERSION <= 30003,
-                           time_to_remove_non_class_ref_support);
-
-  BOOL fixProto3Optional = (flags & GPBDescriptorInitializationFlag_Proto3OptionalKnown) == 0;
-  GPBInternalCompileAssert(GOOGLE_PROTOBUF_OBJC_MIN_SUPPORTED_VERSION <= 30004,
-                           time_to_remove_proto3_optional_fallback);
-
-  BOOL fixClosedEnums = (flags & GPBDescriptorInitializationFlag_ClosedEnumSupportKnown) == 0;
-  GPBInternalCompileAssert(GOOGLE_PROTOBUF_OBJC_MIN_SUPPORTED_VERSION <= 30005,
-                           time_to_remove_closed_enum_fallback);
-
-  if (fixClassRefs || fixProto3Optional || fixClosedEnums) {
-    BOOL fieldsIncludeDefault = (flags & GPBDescriptorInitializationFlag_FieldsWithDefault) != 0;
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wdeprecated-declarations"
-    GPBFileSyntax fileSyntax = file.syntax;
-#pragma clang diagnostic pop
-
-    for (uint32_t i = 0; i < fieldCount; ++i) {
-      GPBMessageFieldDescription *coreDesc;
-      if (fieldsIncludeDefault) {
-        coreDesc = &((((GPBMessageFieldDescriptionWithDefault *)fieldDescriptions)[i]).core);
-      } else {
-        coreDesc = &(((GPBMessageFieldDescription *)fieldDescriptions)[i]);
-      }
-
-      if (fixClassRefs && GPBDataTypeIsMessage(coreDesc->dataType)) {
-        const char *className = coreDesc->dataTypeSpecific.className;
-        Class msgClass = objc_getClass(className);
-        NSAssert(msgClass, @"Class %s not defined", className);
-        coreDesc->dataTypeSpecific.clazz = msgClass;
-      }
-
-      if (fixProto3Optional) {
-        // If it was...
-        //  - proto3 syntax
-        //  - not repeated/map
-        //  - not in a oneof (negative has index)
-        //  - not a message (the flag doesn't make sense for messages)
-        BOOL clearOnZero = ((fileSyntax == GPBFileSyntaxProto3) &&
-                            ((coreDesc->flags & (GPBFieldRepeated | GPBFieldMapKeyMask)) == 0) &&
-                            (coreDesc->hasIndex >= 0) && !GPBDataTypeIsMessage(coreDesc->dataType));
-        if (clearOnZero) {
-          coreDesc->flags |= GPBFieldClearHasIvarOnZero;
-        }
-      }
-
-      if (fixClosedEnums) {
-        // NOTE: This isn't correct, it is using the syntax of the file that
-        // declared the field, not the syntax of the file that declared the
-        // enum; but for older generated code, that's all we have and that happens
-        // to be what the runtime was doing (even though it was wrong). This is
-        // only wrong in the rare cases an enum is declared in a proto3 syntax
-        // file but used for a field in the proto2 syntax file.
-        BOOL isClosedEnum =
-            (coreDesc->dataType == GPBDataTypeEnum && fileSyntax == GPBFileSyntaxProto2);
-        if (isClosedEnum) {
-          coreDesc->flags |= GPBFieldClosedEnum;
-        }
-      }
-    }
-    flags |= (GPBDescriptorInitializationFlag_UsesClassRefs |
-              GPBDescriptorInitializationFlag_Proto3OptionalKnown |
-              GPBDescriptorInitializationFlag_ClosedEnumSupportKnown);
-  }
-
-  GPBDescriptor *result = [self allocDescriptorForClass:messageClass
-                                            messageName:nil
-                                        fileDescription:NULL
-                                                 fields:fieldDescriptions
-                                             fieldCount:fieldCount
-                                            storageSize:storageSize
-                                                  flags:flags];
-  objc_setAssociatedObject(result, &kFileDescriptorCacheKey, file,
-                           OBJC_ASSOCIATION_RETAIN_NONATOMIC);
-  return result;
-}
-
-+ (instancetype)allocDescriptorForClass:(Class)messageClass
-                              rootClass:(__unused Class)rootClass
-                                   file:(GPBFileDescriptor *)file
-                                 fields:(void *)fieldDescriptions
-                             fieldCount:(uint32_t)fieldCount
-                            storageSize:(uint32_t)storageSize
-                                  flags:(GPBDescriptorInitializationFlags)flags {
-  GPBInternalCompileAssert(GOOGLE_PROTOBUF_OBJC_MIN_SUPPORTED_VERSION <= 30006,
-                           time_to_remove_this_old_version_shim);
-  // The rootClass is no longer used, but it is passed as [ROOT class] to
-  // ensure it was started up during initialization also when the message
-  // scopes extensions.
-  return [self allocDescriptorForClass:messageClass
-                                  file:file
-                                fields:fieldDescriptions
-                            fieldCount:fieldCount
-                           storageSize:storageSize
-                                 flags:flags];
-}
-
 - (instancetype)initWithClass:(Class)messageClass
                   messageName:(NSString *)messageName
               fileDescription:(GPBFileDescription *)fileDescription
@@ -304,7 +203,8 @@ static NSArray *NewFieldsArrayForHasIndex(int hasIndex, NSArray *allMessageField
       firstHasIndex:(int32_t)firstHasIndex {
   NSCAssert(firstHasIndex < 0, @"Should always be <0");
   NSMutableArray *oneofs = [[NSMutableArray alloc] initWithCapacity:count];
-  for (uint32_t i = 0, hasIndex = firstHasIndex; i < count; ++i, --hasIndex) {
+  int32_t hasIndex = firstHasIndex;
+  for (uint32_t i = 0; i < count; ++i, --hasIndex) {
     const char *name = oneofNames[i];
     NSArray *fieldsForOneof = NewFieldsArrayForHasIndex(hasIndex, fields_);
     NSCAssert(fieldsForOneof.count > 0, @"No fields for this oneof? (%s:%d)", name, hasIndex);
@@ -332,32 +232,13 @@ static NSArray *NewFieldsArrayForHasIndex(int hasIndex, NSArray *allMessageField
 
 - (void)setupExtensionRanges:(const GPBExtensionRange *)ranges count:(int32_t)count {
   extensionRanges_ = ranges;
-  extensionRangesCount_ = count;
+  extensionRangesCount_ = (uint32_t)count;
 }
 
 - (void)setupContainingMessageClass:(Class)messageClass {
   objc_setAssociatedObject(self, &kParentClassValueKey, messageClass, OBJC_ASSOCIATION_ASSIGN);
 }
 
-- (void)setupContainingMessageClassName:(const char *)msgClassName {
-  GPBInternalCompileAssert(GOOGLE_PROTOBUF_OBJC_MIN_SUPPORTED_VERSION <= 30003,
-                           time_to_remove_this_old_version_shim);
-  // Note: Only fetch the class here, can't send messages to it because
-  // that could cause cycles back to this class within +initialize if
-  // two messages have each other in fields (i.e. - they build a graph).
-  Class clazz = objc_getClass(msgClassName);
-  NSAssert(clazz, @"Class %s not defined", msgClassName);
-  [self setupContainingMessageClass:clazz];
-}
-
-- (void)setupMessageClassNameSuffix:(NSString *)suffix {
-  GPBInternalCompileAssert(GOOGLE_PROTOBUF_OBJC_MIN_SUPPORTED_VERSION <= 30007,
-                           time_to_remove_this_old_version_shim);
-  if (suffix.length) {
-    objc_setAssociatedObject(self, &kClassNameSuffixKey, suffix, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
-  }
-}
-
 - (NSString *)name {
   return NSStringFromClass(messageClass_);
 }
@@ -373,12 +254,10 @@ static NSArray *NewFieldsArrayForHasIndex(int hasIndex, NSArray *allMessageField
       NSString *package = fileDescription_->package ? @(fileDescription_->package) : @"";
       if (fileDescription_->prefix) {
         result = [[GPBFileDescriptor alloc] initWithPackage:package
-                                                 objcPrefix:@(fileDescription_->prefix)
-                                                     syntax:fileDescription_->syntax];
+                                                 objcPrefix:@(fileDescription_->prefix)];
 
       } else {
-        result = [[GPBFileDescriptor alloc] initWithPackage:package
-                                                     syntax:fileDescription_->syntax];
+        result = [[GPBFileDescriptor alloc] initWithPackage:package];
       }
       objc_setAssociatedObject(result, &kFileDescriptorCacheKey, result,
                                OBJC_ASSOCIATION_RETAIN_NONATOMIC);
@@ -404,62 +283,10 @@ static NSArray *NewFieldsArrayForHasIndex(int hasIndex, NSArray *allMessageField
     return messageName_;
   }
 
-  GPBInternalCompileAssert(GOOGLE_PROTOBUF_OBJC_MIN_SUPPORTED_VERSION <= 30007,
-                           time_to_remove_this_old_approach);
-  // NOTE: When this code path is removed, this also means this api can't return nil any more but
-  // that would be a breaking code change (not longer a Swift optional), so changing that will be
-  // harder.
-
-  NSString *className = NSStringFromClass(self.messageClass);
-  GPBFileDescriptor *file = self.file;
-  NSString *objcPrefix = file.objcPrefix;
-  if (objcPrefix && ![className hasPrefix:objcPrefix]) {
-    NSAssert(0, @"Class didn't have correct prefix? (%@ - %@)", className, objcPrefix);
-    return nil;
-  }
-
-  NSString *name = nil;
-  if (parent) {
-    NSString *parentClassName = NSStringFromClass(parent.messageClass);
-    // The generator will add _Class to avoid reserved words, drop it.
-    NSString *suffix = objc_getAssociatedObject(parent, &kClassNameSuffixKey);
-    if (suffix) {
-      if (![parentClassName hasSuffix:suffix]) {
-        NSAssert(0, @"ParentMessage class didn't have correct suffix? (%@ - %@)", className,
-                 suffix);
-        return nil;
-      }
-      parentClassName = [parentClassName substringToIndex:(parentClassName.length - suffix.length)];
-    }
-    NSString *parentPrefix = [parentClassName stringByAppendingString:@"_"];
-    if (![className hasPrefix:parentPrefix]) {
-      NSAssert(0, @"Class didn't have the correct parent name prefix? (%@ - %@)", parentPrefix,
-               className);
-      return nil;
-    }
-    name = [className substringFromIndex:parentPrefix.length];
-  } else {
-    name = [className substringFromIndex:objcPrefix.length];
-  }
-
-  // The generator will add _Class to avoid reserved words, drop it.
-  NSString *suffix = objc_getAssociatedObject(self, &kClassNameSuffixKey);
-  if (suffix) {
-    if (![name hasSuffix:suffix]) {
-      NSAssert(0, @"Message class didn't have correct suffix? (%@ - %@)", name, suffix);
-      return nil;
-    }
-    name = [name substringToIndex:(name.length - suffix.length)];
-  }
-
-  NSString *prefix = (parent != nil ? parent.fullName : file.package);
-  NSString *result;
-  if (prefix.length > 0) {
-    result = [NSString stringWithFormat:@"%@.%@", prefix, name];
-  } else {
-    result = name;
-  }
-  return result;
+#if defined(DEBUG) && DEBUG
+  NSAssert(NO, @"Missing messageName_");
+#endif
+  return nil;
 }
 
 - (GPBFieldDescriptor *)fieldWithNumber:(uint32_t)fieldNumber {
@@ -494,30 +321,24 @@ static NSArray *NewFieldsArrayForHasIndex(int hasIndex, NSArray *allMessageField
 @implementation GPBFileDescriptor {
   NSString *package_;
   NSString *objcPrefix_;
-  GPBFileSyntax syntax_;
 }
 
 @synthesize package = package_;
 @synthesize objcPrefix = objcPrefix_;
-@synthesize syntax = syntax_;
 
-- (instancetype)initWithPackage:(NSString *)package
-                     objcPrefix:(NSString *)objcPrefix
-                         syntax:(GPBFileSyntax)syntax {
+- (instancetype)initWithPackage:(NSString *)package objcPrefix:(NSString *)objcPrefix {
   self = [super init];
   if (self) {
     package_ = [package copy];
     objcPrefix_ = [objcPrefix copy];
-    syntax_ = syntax;
   }
   return self;
 }
 
-- (instancetype)initWithPackage:(NSString *)package syntax:(GPBFileSyntax)syntax {
+- (instancetype)initWithPackage:(NSString *)package {
   self = [super init];
   if (self) {
     package_ = [package copy];
-    syntax_ = syntax;
   }
   return self;
 }
@@ -537,7 +358,7 @@ static NSArray *NewFieldsArrayForHasIndex(int hasIndex, NSArray *allMessageField
   }
   GPBFileDescriptor *otherFile = other;
   // objcPrefix can be nil, otherwise, straight up compare.
-  return (syntax_ == otherFile->syntax_ && [package_ isEqual:otherFile->package_] &&
+  return ([package_ isEqual:otherFile->package_] &&
           (objcPrefix_ == otherFile->objcPrefix_ ||
            (otherFile->objcPrefix_ && [objcPrefix_ isEqual:otherFile->objcPrefix_])));
 }
@@ -669,9 +490,17 @@ uint32_t GPBFieldAlternateTag(GPBFieldDescriptor *self) {
       NSAssert((coreDesc->flags & GPBFieldHasEnumDescriptor) != 0,
                @"Field must have GPBFieldHasEnumDescriptor set");
 #endif  // DEBUG
+#if defined(DEBUG) && DEBUG && !defined(NS_BLOCK_ASSERTIONS)
+      if (enumDescriptor_.isClosed) {
+        NSAssert((coreDesc->flags & GPBFieldClosedEnum) != 0,
+                 @"Field must have GPBFieldClosedEnum set");
+      } else {
+        NSAssert((coreDesc->flags & GPBFieldClosedEnum) == 0,
+                 @"Field must not have GPBFieldClosedEnum set");
+      }
+#endif  // defined(DEBUG) && DEBUG && !NS_BLOCK_ASSERTIONS
     }
 
-    // Non map<>/repeated fields can have defaults in proto2 syntax.
     BOOL isMapOrArray = GPBFieldIsMapOrArray(self);
     if (!isMapOrArray && includesDefault) {
       defaultValue_ = ((GPBMessageFieldDescriptionWithDefault *)description)->defaultValue;
@@ -813,7 +642,7 @@ uint32_t GPBFieldAlternateTag(GPBFieldDescriptor *self) {
       return nil;
     }
     const uint8_t *extraTextFormatInfo = [extraInfoValue pointerValue];
-    return GPBDecodeTextFormatName(extraTextFormatInfo, GPBFieldNumber(self), self.name);
+    return GPBDecodeTextFormatName(extraTextFormatInfo, (int32_t)GPBFieldNumber(self), self.name);
   }
 
   // The logic here has to match SetCommonFieldVariables() from
@@ -895,7 +724,7 @@ uint32_t GPBFieldAlternateTag(GPBFieldDescriptor *self) {
   // (from the generated code) to detect when sources from a newer version are being used with an
   // older runtime.
   GPBEnumDescriptorInitializationFlags unknownFlags =
-      ~(GPBEnumDescriptorInitializationFlag_IsClosed);
+      (GPBEnumDescriptorInitializationFlags)(~(GPBEnumDescriptorInitializationFlag_IsClosed));
   if ((flags & unknownFlags) != 0) {
     GPBRuntimeMatchFailure();
   }
@@ -927,38 +756,6 @@ uint32_t GPBFieldAlternateTag(GPBFieldDescriptor *self) {
   return descriptor;
 }
 
-+ (instancetype)allocDescriptorForName:(NSString *)name
-                            valueNames:(const char *)valueNames
-                                values:(const int32_t *)values
-                                 count:(uint32_t)valueCount
-                          enumVerifier:(GPBEnumValidationFunc)enumVerifier {
-  GPBInternalCompileAssert(GOOGLE_PROTOBUF_OBJC_MIN_SUPPORTED_VERSION <= 30005,
-                           time_to_remove_this_old_version_shim);
-  return [self allocDescriptorForName:name
-                           valueNames:valueNames
-                               values:values
-                                count:valueCount
-                         enumVerifier:enumVerifier
-                                flags:GPBEnumDescriptorInitializationFlag_None];
-}
-
-+ (instancetype)allocDescriptorForName:(NSString *)name
-                            valueNames:(const char *)valueNames
-                                values:(const int32_t *)values
-                                 count:(uint32_t)valueCount
-                          enumVerifier:(GPBEnumValidationFunc)enumVerifier
-                   extraTextFormatInfo:(const char *)extraTextFormatInfo {
-  GPBInternalCompileAssert(GOOGLE_PROTOBUF_OBJC_MIN_SUPPORTED_VERSION <= 30005,
-                           time_to_remove_this_old_version_shim);
-  return [self allocDescriptorForName:name
-                           valueNames:valueNames
-                               values:values
-                                count:valueCount
-                         enumVerifier:enumVerifier
-                                flags:GPBEnumDescriptorInitializationFlag_None
-                  extraTextFormatInfo:extraTextFormatInfo];
-}
-
 - (instancetype)initWithName:(NSString *)name
                   valueNames:(const char *)valueNames
                       values:(const int32_t *)values
@@ -993,6 +790,10 @@ uint32_t GPBFieldAlternateTag(GPBFieldDescriptor *self) {
   return (flags_ & GPBEnumDescriptorInitializationFlag_IsClosed) != 0;
 }
 
+- (BOOL)isOpenOrValidValue:(int32_t)value {
+  return (flags_ & GPBEnumDescriptorInitializationFlag_IsClosed) == 0 || enumVerifier_(value);
+}
+
 - (void)calcValueNameOffsets {
   @synchronized(self) {
     if (nameOffsets_ != NULL) {
@@ -1140,8 +941,8 @@ uint32_t GPBFieldAlternateTag(GPBFieldDescriptor *self) {
   // Compute the unknown options by this version of the runtime and then check the passed in
   // descriptor's options (from the generated code) to detect when sources from a newer version are
   // being used with an older runtime.
-  GPBExtensionOptions unknownOptions =
-      ~(GPBExtensionRepeated | GPBExtensionPacked | GPBExtensionSetWireFormat);
+  GPBExtensionOptions unknownOptions = (GPBExtensionOptions)(~(
+      GPBExtensionRepeated | GPBExtensionPacked | GPBExtensionSetWireFormat));
   if ((desc->options & unknownOptions) != 0) {
     GPBRuntimeMatchFailure();
   }
@@ -1186,27 +987,6 @@ uint32_t GPBFieldAlternateTag(GPBFieldDescriptor *self) {
   return self;
 }
 
-- (instancetype)initWithExtensionDescription:(GPBExtensionDescription *)desc {
-  GPBInternalCompileAssert(GOOGLE_PROTOBUF_OBJC_MIN_SUPPORTED_VERSION <= 30003,
-                           time_to_remove_this_old_version_shim);
-
-  const char *className = desc->messageOrGroupClass.name;
-  if (className) {
-    Class clazz = objc_lookUpClass(className);
-    NSAssert(clazz != Nil, @"Class %s not defined", className);
-    desc->messageOrGroupClass.clazz = clazz;
-  }
-
-  const char *extendedClassName = desc->extendedClass.name;
-  if (extendedClassName) {
-    Class clazz = objc_lookUpClass(extendedClassName);
-    NSAssert(clazz, @"Class %s not defined", extendedClassName);
-    desc->extendedClass.clazz = clazz;
-  }
-
-  return [self initWithExtensionDescription:desc usesClassRefs:YES];
-}
-
 - (void)dealloc {
   if ((description_->dataType == GPBDataTypeBytes) && !GPBExtensionIsRepeated(description_)) {
     [defaultValue_.valueData release];
@@ -1230,7 +1010,7 @@ uint32_t GPBFieldAlternateTag(GPBFieldDescriptor *self) {
 }
 
 - (uint32_t)fieldNumber {
-  return description_->fieldNumber;
+  return (uint32_t)(description_->fieldNumber);
 }
 
 - (GPBDataType)dataType {
