@@ -5,11 +5,13 @@
 // license that can be found in the LICENSE file or at
 // https://developers.google.com/open-source/licenses/bsd
 
-#import "GPBTestUtilities.h"
-
 #import "GPBCodedInputStream.h"
 #import "GPBMessage_PackagePrivate.h"
+#import "GPBTestUtilities.h"
+#import "GPBUnknownField.h"
 #import "GPBUnknownField_PackagePrivate.h"
+#import "GPBUnknownFields.h"
+#import "GPBWireFormat.h"
 #import "objectivec/Tests/Unittest.pbobjc.h"
 #import "objectivec/Tests/UnittestMset.pbobjc.h"
 
@@ -110,32 +112,36 @@
 }
 
 const int kUnknownTypeId = 1550055;
+const int kUnknownTypeId2 = 1550056;
 
 - (void)testSerializeMessageSet {
   // Set up a MSetMessage with two known messages and an unknown one.
   MSetMessage* message_set = [MSetMessage message];
   [[message_set getExtension:[MSetMessageExtension1 messageSetExtension]] setI:123];
   [[message_set getExtension:[MSetMessageExtension2 messageSetExtension]] setStr:@"foo"];
-  GPBUnknownField* unknownField =
-      [[[GPBUnknownField alloc] initWithNumber:kUnknownTypeId] autorelease];
-  [unknownField addLengthDelimited:[NSData dataWithBytes:"bar" length:3]];
-  GPBUnknownFieldSet* unknownFieldSet = [[[GPBUnknownFieldSet alloc] init] autorelease];
-  [unknownFieldSet addField:unknownField];
-  [message_set setUnknownFields:unknownFieldSet];
+
+  GPBUnknownFields* ufs = [[[GPBUnknownFields alloc] init] autorelease];
+  GPBUnknownFields* group = [ufs addGroupWithFieldNumber:GPBWireFormatMessageSetItem];
+  [group addFieldNumber:GPBWireFormatMessageSetTypeId varint:kUnknownTypeId2];
+  [group addFieldNumber:GPBWireFormatMessageSetMessage lengthDelimited:DataFromCStr("baz")];
+  XCTAssertTrue([message_set mergeUnknownFields:ufs
+                              extensionRegistry:[MSetUnittestMsetRoot extensionRegistry]
+                                          error:NULL]);
 
   NSData* data = [message_set data];
 
   // Parse back using MSetRawMessageSet and check the contents.
   MSetRawMessageSet* raw = [MSetRawMessageSet parseFromData:data error:NULL];
 
-  XCTAssertEqual([raw.unknownFields countOfFields], (NSUInteger)0);
+  GPBUnknownFields* ufs2 = [[[GPBUnknownFields alloc] initFromMessage:raw] autorelease];
+  XCTAssertTrue(ufs2.empty);
 
   XCTAssertEqual(raw.itemArray.count, (NSUInteger)3);
   XCTAssertEqual((uint32_t)[raw.itemArray[0] typeId],
                  [MSetMessageExtension1 messageSetExtension].fieldNumber);
   XCTAssertEqual((uint32_t)[raw.itemArray[1] typeId],
                  [MSetMessageExtension2 messageSetExtension].fieldNumber);
-  XCTAssertEqual([raw.itemArray[2] typeId], kUnknownTypeId);
+  XCTAssertEqual([raw.itemArray[2] typeId], kUnknownTypeId2);
 
   MSetMessageExtension1* message1 =
       [MSetMessageExtension1 parseFromData:[((MSetRawMessageSet_Item*)raw.itemArray[0]) message]
@@ -147,7 +153,7 @@ const int kUnknownTypeId = 1550055;
                                      error:NULL];
   XCTAssertEqualObjects(message2.str, @"foo");
 
-  XCTAssertEqualObjects([raw.itemArray[2] message], [NSData dataWithBytes:"bar" length:3]);
+  XCTAssertEqualObjects([raw.itemArray[2] message], DataFromCStr("baz"));
 }
 
 - (void)testParseMessageSet {
@@ -175,7 +181,7 @@ const int kUnknownTypeId = 1550055;
   {
     MSetRawMessageSet_Item* item = [MSetRawMessageSet_Item message];
     item.typeId = kUnknownTypeId;
-    item.message = [NSData dataWithBytes:"bar" length:3];
+    item.message = DataFromCStr("bar");
     [raw.itemArray addObject:item];
   }
 
@@ -190,11 +196,94 @@ const int kUnknownTypeId = 1550055;
   XCTAssertEqualObjects([[messageSet getExtension:[MSetMessageExtension2 messageSetExtension]] str],
                         @"foo");
 
-  XCTAssertEqual([messageSet.unknownFields countOfFields], (NSUInteger)1);
-  GPBUnknownField* unknownField = [messageSet.unknownFields getField:kUnknownTypeId];
-  XCTAssertNotNil(unknownField);
-  XCTAssertEqual(unknownField.lengthDelimitedList.count, (NSUInteger)1);
-  XCTAssertEqualObjects(unknownField.lengthDelimitedList[0], [NSData dataWithBytes:"bar" length:3]);
+  GPBUnknownFields* ufs = [[[GPBUnknownFields alloc] initFromMessage:messageSet] autorelease];
+  XCTAssertEqual(ufs.count, (NSUInteger)1);
+  GPBUnknownFields* group = [ufs firstGroup:GPBWireFormatMessageSetItem];
+  XCTAssertNotNil(group);
+  XCTAssertEqual(group.count, (NSUInteger)2);
+  uint64_t varint = 0;
+  XCTAssertTrue([group getFirst:GPBWireFormatMessageSetTypeId varint:&varint]);
+  XCTAssertEqual(varint, kUnknownTypeId);
+  XCTAssertEqualObjects([group firstLengthDelimited:GPBWireFormatMessageSetMessage],
+                        DataFromCStr("bar"));
+}
+
+- (void)testParseMessageSet_FirstValueSticks {
+  MSetRawBreakableMessageSet* raw = [MSetRawBreakableMessageSet message];
+
+  {
+    MSetRawBreakableMessageSet_Item* item = [MSetRawBreakableMessageSet_Item message];
+
+    [item.typeIdArray addValue:[MSetMessageExtension1 messageSetExtension].fieldNumber];
+    MSetMessageExtension1* message1 = [MSetMessageExtension1 message];
+    message1.i = 123;
+    NSData* itemData = [message1 data];
+    [item.messageArray addObject:itemData];
+
+    [item.typeIdArray addValue:[MSetMessageExtension2 messageSetExtension].fieldNumber];
+    MSetMessageExtension2* message2 = [MSetMessageExtension2 message];
+    message2.str = @"foo";
+    itemData = [message2 data];
+    [item.messageArray addObject:itemData];
+
+    [raw.itemArray addObject:item];
+  }
+
+  NSData* data = [raw data];
+
+  // Parse as a MSetMessage and check the contents.
+  NSError* err = nil;
+  MSetMessage* messageSet = [MSetMessage parseFromData:data
+                                     extensionRegistry:[MSetUnittestMsetRoot extensionRegistry]
+                                                 error:&err];
+  XCTAssertNotNil(messageSet);
+  XCTAssertNil(err);
+  XCTAssertTrue([messageSet hasExtension:[MSetMessageExtension1 messageSetExtension]]);
+  XCTAssertEqual([[messageSet getExtension:[MSetMessageExtension1 messageSetExtension]] i], 123);
+  XCTAssertFalse([messageSet hasExtension:[MSetMessageExtension2 messageSetExtension]]);
+  GPBUnknownFields* ufs = [[[GPBUnknownFields alloc] initFromMessage:messageSet] autorelease];
+  XCTAssertTrue(ufs.empty);
+}
+
+- (void)testParseMessageSet_PartialValuesDropped {
+  MSetRawBreakableMessageSet* raw = [MSetRawBreakableMessageSet message];
+
+  {
+    MSetRawBreakableMessageSet_Item* item = [MSetRawBreakableMessageSet_Item message];
+    [item.typeIdArray addValue:[MSetMessageExtension1 messageSetExtension].fieldNumber];
+    // No payload.
+    [raw.itemArray addObject:item];
+  }
+
+  {
+    MSetRawBreakableMessageSet_Item* item = [MSetRawBreakableMessageSet_Item message];
+    // No type ID.
+    MSetMessageExtension2* message = [MSetMessageExtension2 message];
+    message.str = @"foo";
+    NSData* itemData = [message data];
+    [item.messageArray addObject:itemData];
+    [raw.itemArray addObject:item];
+  }
+
+  {
+    MSetRawBreakableMessageSet_Item* item = [MSetRawBreakableMessageSet_Item message];
+    // Neither type ID nor payload.
+    [raw.itemArray addObject:item];
+  }
+
+  NSData* data = [raw data];
+
+  // Parse as a MSetMessage and check the contents.
+  NSError* err = nil;
+  MSetMessage* messageSet = [MSetMessage parseFromData:data
+                                     extensionRegistry:[MSetUnittestMsetRoot extensionRegistry]
+                                                 error:&err];
+  XCTAssertNotNil(messageSet);
+  XCTAssertNil(err);
+  XCTAssertEqual([messageSet extensionsCurrentlySet].count,
+                 (NSUInteger)0);  // None because they were all partial and dropped.
+  GPBUnknownFields* ufs = [[[GPBUnknownFields alloc] initFromMessage:messageSet] autorelease];
+  XCTAssertTrue(ufs.empty);
 }
 
 - (void)assertFieldsInOrder:(NSData*)data {
