@@ -1,32 +1,9 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 // Author: anuraag@google.com (Anuraag Agrawal)
 // Author: tibell@google.com (Johan Tibell)
@@ -316,7 +293,7 @@ static PyObject* RichCompare(PyObject* pself, PyObject* other, int opid) {
     return nullptr;
   }
   if (opid == Py_EQ || opid == Py_NE) {
-    // TODO(anuraag): Don't make new lists just for this...
+    // TODO: Don't make new lists just for this...
     ScopedPyObjectPtr full_slice(PySlice_New(nullptr, nullptr, nullptr));
     if (full_slice == nullptr) {
       return nullptr;
@@ -549,22 +526,26 @@ PyTypeObject RepeatedCompositeContainer_Type = {
 #if PY_VERSION_HEX >= 0x03080000
     0,  //  tp_vectorcall_offset
 #else
-    nullptr,  //  tp_print
+    nullptr,             //  tp_print
+#endif
+    nullptr,                                   //  tp_getattr
+    nullptr,                                   //  tp_setattr
+    nullptr,                                   //  tp_compare
+    repeated_composite_container::ToStr,       //  tp_repr
+    nullptr,                                   //  tp_as_number
+    &repeated_composite_container::SqMethods,  //  tp_as_sequence
+    &repeated_composite_container::MpMethods,  //  tp_as_mapping
+    PyObject_HashNotImplemented,               //  tp_hash
+    nullptr,                                   //  tp_call
+    nullptr,                                   //  tp_str
+    nullptr,                                   //  tp_getattro
+    nullptr,                                   //  tp_setattro
+    nullptr,                                   //  tp_as_buffer
+#if PY_VERSION_HEX >= 0x030A0000
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_SEQUENCE,  //  tp_flags
+#else
+    Py_TPFLAGS_DEFAULT,  //  tp_flags
 #endif
-    nullptr,                                    //  tp_getattr
-    nullptr,                                    //  tp_setattr
-    nullptr,                                    //  tp_compare
-    repeated_composite_container::ToStr,        //  tp_repr
-    nullptr,                                    //  tp_as_number
-    &repeated_composite_container::SqMethods,   //  tp_as_sequence
-    &repeated_composite_container::MpMethods,   //  tp_as_mapping
-    PyObject_HashNotImplemented,                //  tp_hash
-    nullptr,                                    //  tp_call
-    nullptr,                                    //  tp_str
-    nullptr,                                    //  tp_getattro
-    nullptr,                                    //  tp_setattro
-    nullptr,                                    //  tp_as_buffer
-    Py_TPFLAGS_DEFAULT,                         //  tp_flags
     "A Repeated scalar container",              //  tp_doc
     nullptr,                                    //  tp_traverse
     nullptr,                                    //  tp_clear
