@@ -7,6 +7,7 @@
 
 use bad_names_rust_proto::*;
 use googletest::prelude::*;
+use protobuf::proto;
 
 #[gtest]
 fn test_reserved_keyword_in_accessors() {
@@ -21,6 +22,14 @@ fn test_reserved_keyword_in_messages() {
     let _ = Ref::new().r#const();
 }
 
+#[gtest]
+fn test_reserved_keyword_with_proto_macro() {
+    let _ = proto!(Self__mangled_because_ident_isnt_a_legal_raw_identifier {
+        r#true: false,
+        r#match: [0i32],
+    });
+}
+
 #[gtest]
 fn test_collision_in_accessors() {
     let mut m = AccessorsCollide::new();
