@@ -45,7 +45,11 @@ absl::flat_hash_map<absl::string_view, std::string> EnumVars(
   auto classname = ClassName(enum_, false);
   return {
       {"Enum", std::string(enum_->name())},
-      {"Enum_", ResolveKeyword(enum_->name())},
+      {"Enum_", ResolveKnownNameCollisions(enum_->name(),
+                                           enum_->containing_type() != nullptr
+                                               ? NameContext::kMessage
+                                               : NameContext::kFile,
+                                           NameKind::kType)},
       {"Msg_Enum", classname},
       {"::Msg_Enum", QualifiedClassName(enum_, options)},
       {"Msg_Enum_",
@@ -104,6 +108,16 @@ EnumGenerator::EnumGenerator(const EnumDescriptor* descriptor,
   size_t total_values = static_cast<size_t>(enum_->value_count());
   should_cache_ = has_reflection_ &&
                   (values_range < 16u || values_range < total_values * 2u);
+
+  sorted_unique_values_.reserve(enum_->value_count());
+  for (int i = 0; i < enum_->value_count(); ++i) {
+    sorted_unique_values_.push_back(enum_->value(i)->number());
+  }
+  // Sort and deduplicate
+  absl::c_sort(sorted_unique_values_);
+  sorted_unique_values_.erase(
+      std::unique(sorted_unique_values_.begin(), sorted_unique_values_.end()),
+      sorted_unique_values_.end());
 }
 
 void EnumGenerator::GenerateDefinition(io::Printer* p) {
@@ -167,27 +181,31 @@ void EnumGenerator::GenerateDefinition(io::Printer* p) {
           $open_enum_sentinels$,
         };
 
-        $dllexport_decl $bool $Msg_Enum$_IsValid(int value);
         $dllexport_decl $extern const uint32_t $Msg_Enum$_internal_data_[];
-        constexpr $Msg_Enum$ $Msg_Enum_Enum_MIN$ = static_cast<$Msg_Enum$>($kMin$);
-        constexpr $Msg_Enum$ $Msg_Enum_Enum_MAX$ = static_cast<$Msg_Enum$>($kMax$);
+        inline constexpr $Msg_Enum$ $Msg_Enum_Enum_MIN$ =
+            static_cast<$Msg_Enum$>($kMin$);
+        inline constexpr $Msg_Enum$ $Msg_Enum_Enum_MAX$ =
+            static_cast<$Msg_Enum$>($kMax$);
       )cc");
 
+  // Generate the inline `_IsValid` function choosing the best implementation
+  // for the values.
+  GenerateIsValid(p);
+
   if (generate_array_size_) {
     p->Emit({Sub("Msg_Enum_Enum_ARRAYSIZE",
                  absl::StrCat(p->LookupVar("Msg_Enum_"), enum_->name(),
                               "_ARRAYSIZE"))
                  .AnnotatedAs(enum_)},
             R"cc(
-              constexpr int $Msg_Enum_Enum_ARRAYSIZE$ = $kMax$ + 1;
+              inline constexpr int $Msg_Enum_Enum_ARRAYSIZE$ = $kMax$ + 1;
             )cc");
   }
 
   if (has_reflection_) {
-    p->Emit(R"cc(
-      $dllexport_decl $const ::$proto_ns$::EnumDescriptor*
-      $Msg_Enum$_descriptor();
-    )cc");
+    p->Emit(R"(
+      $dllexport_decl $const $pb$::EnumDescriptor* $nonnull$ $Msg_Enum$_descriptor();
+    )");
   } else {
     p->Emit(R"cc(
       $return_type$ $Msg_Enum$_Name($Msg_Enum$ value);
@@ -225,8 +243,7 @@ void EnumGenerator::GenerateDefinition(io::Printer* p) {
       p->Emit(R"cc(
         template <>
         inline $return_type$ $Msg_Enum$_Name($Msg_Enum$ value) {
-          return ::$proto_ns$::internal::NameOfDenseEnum<$Msg_Enum$_descriptor,
-                                                         $kMin$, $kMax$>(
+          return $pbi$::NameOfDenseEnum<$Msg_Enum$_descriptor, $kMin$, $kMax$>(
               static_cast<int>(value));
         }
       )cc");
@@ -236,21 +253,25 @@ void EnumGenerator::GenerateDefinition(io::Printer* p) {
       template <typename T>
       $return_type$ $Msg_Enum$_Name(T value) {
         $static_assert$;
-        return ::$proto_ns$::internal::NameOfEnum($Msg_Enum$_descriptor(), value);
+        return $pbi$::NameOfEnum($Msg_Enum$_descriptor(), value);
       }
     )cc");
   }
 
   if (has_reflection_) {
     p->Emit(R"cc(
-      inline bool $Msg_Enum$_Parse(absl::string_view name, $Msg_Enum$* value) {
-        return ::$proto_ns$::internal::ParseNamedEnum<$Msg_Enum$>(
-            $Msg_Enum$_descriptor(), name, value);
+      inline bool $Msg_Enum$_Parse(
+          //~
+          absl::string_view name, $Msg_Enum$* $nonnull$ value) {
+        return $pbi$::ParseNamedEnum<$Msg_Enum$>($Msg_Enum$_descriptor(), name,
+                                                 value);
       }
     )cc");
   } else {
     p->Emit(R"cc(
-      bool $Msg_Enum$_Parse(absl::string_view name, $Msg_Enum$* value);
+      bool $Msg_Enum$_Parse(
+          //~
+          absl::string_view name, $Msg_Enum$* $nonnull$ value);
     )cc");
   }
 }
@@ -267,7 +288,7 @@ void EnumGenerator::GenerateGetEnumDescriptorSpecializations(io::Printer* p) {
   }
   p->Emit(R"cc(
     template <>
-    inline const EnumDescriptor* GetEnumDescriptor<$::Msg_Enum$>() {
+    inline const EnumDescriptor* $nonnull$ GetEnumDescriptor<$::Msg_Enum$>() {
       return $::Msg_Enum$_descriptor();
     }
   )cc");
@@ -321,11 +342,11 @@ void EnumGenerator::GenerateSymbolImports(io::Printer* p) const {
   }
 
   if (has_reflection_) {
-    p->Emit(R"cc(
-      static inline const ::$proto_ns$::EnumDescriptor* $Enum$_descriptor() {
+    p->Emit(R"(
+      static inline const $pb$::EnumDescriptor* $nonnull$ $Enum$_descriptor() {
         return $Msg_Enum$_descriptor();
       }
-    )cc");
+    )");
   }
 
   p->Emit(R"cc(
@@ -333,71 +354,42 @@ void EnumGenerator::GenerateSymbolImports(io::Printer* p) const {
     static inline $return_type$ $Enum$_Name(T value) {
       return $Msg_Enum$_Name(value);
     }
-    static inline bool $Enum$_Parse(absl::string_view name, $Enum_$* value) {
+    static inline bool $Enum$_Parse(
+        //~
+        absl::string_view name, $Enum_$* $nonnull$ value) {
       return $Msg_Enum$_Parse(name, value);
     }
   )cc");
 }
 
-void EnumGenerator::GenerateMethods(int idx, io::Printer* p) {
+void EnumGenerator::GenerateIsValid(io::Printer* p) const {
   auto v = p->WithVars(EnumVars(enum_, options_, limits_.min, limits_.max));
 
-  if (has_reflection_) {
-    p->Emit({{"idx", idx}}, R"cc(
-      const ::$proto_ns$::EnumDescriptor* $Msg_Enum$_descriptor() {
-        ::$proto_ns$::internal::AssignDescriptors(&$desc_table$);
-        return $file_level_enum_descriptors$[$idx$];
-      }
-    )cc");
-  }
-
-  // Multiple values may have the same number. Sort and dedup.
-  std::vector<int> numbers;
-  numbers.reserve(enum_->value_count());
-  for (int i = 0; i < enum_->value_count(); ++i) {
-    numbers.push_back(enum_->value(i)->number());
-  }
-  // Sort and deduplicate `numbers`.
-  absl::c_sort(numbers);
-  numbers.erase(std::unique(numbers.begin(), numbers.end()), numbers.end());
-
-  // We now generate the XXX_IsValid functions, as well as their encoded enum
-  // data.
   // For simple enums we skip the generic ValidateEnum call and use better
   // codegen. It matches the speed of the previous switch-based codegen.
   // For more complex enums we use the new algorithm with the encoded data.
-  // Always generate the data array, even on the simple cases because someone
-  // might be using it for TDP entries. If it is not used in the end, the linker
-  // will drop it.
-  p->Emit({{"encoded",
-            [&] {
-              for (uint32_t n : google::protobuf::internal::GenerateEnumData(numbers)) {
-                p->Emit({{"n", n}}, "$n$u, ");
-              }
-            }}},
-          R"cc(
-            PROTOBUF_CONSTINIT const uint32_t $Msg_Enum$_internal_data_[] = {
-                $encoded$};
-          )cc");
 
-  if (numbers.front() + static_cast<int64_t>(numbers.size()) - 1 ==
-      numbers.back()) {
+  if (sorted_unique_values_.front() +
+          static_cast<int64_t>(sorted_unique_values_.size()) - 1 ==
+      sorted_unique_values_.back()) {
     // They are sequential. Do a simple range check.
-    p->Emit({{"min", numbers.front()}, {"max", numbers.back()}},
+    p->Emit({{"min", sorted_unique_values_.front()},
+             {"max", sorted_unique_values_.back()}},
             R"cc(
-              bool $Msg_Enum$_IsValid(int value) {
+              inline bool $Msg_Enum$_IsValid(int value) {
                 return $min$ <= value && value <= $max$;
               }
             )cc");
-  } else if (numbers.front() >= 0 && numbers.back() < 64) {
+  } else if (sorted_unique_values_.front() >= 0 &&
+             sorted_unique_values_.back() < 64) {
     // Not sequential, but they fit in a 64-bit bitmap.
     uint64_t bitmap = 0;
-    for (int n : numbers) {
+    for (int n : sorted_unique_values_) {
       bitmap |= uint64_t{1} << n;
     }
-    p->Emit({{"bitmap", bitmap}, {"max", numbers.back()}},
+    p->Emit({{"bitmap", bitmap}, {"max", sorted_unique_values_.back()}},
             R"cc(
-              bool $Msg_Enum$_IsValid(int value) {
+              inline bool $Msg_Enum$_IsValid(int value) {
                 return 0 <= value && value <= $max$ && (($bitmap$u >> value) & 1) != 0;
               }
             )cc");
@@ -405,11 +397,39 @@ void EnumGenerator::GenerateMethods(int idx, io::Printer* p) {
     // More complex struct. Use enum data structure for lookup.
     p->Emit(
         R"cc(
-          bool $Msg_Enum$_IsValid(int value) {
-            return ::_pbi::ValidateEnum(value, $Msg_Enum$_internal_data_);
+          inline bool $Msg_Enum$_IsValid(int value) {
+            return $pbi$::ValidateEnum(value, $Msg_Enum$_internal_data_);
           }
         )cc");
   }
+}
+
+void EnumGenerator::GenerateMethods(int idx, io::Printer* p) {
+  auto v = p->WithVars(EnumVars(enum_, options_, limits_.min, limits_.max));
+
+  if (has_reflection_) {
+    p->Emit({{"idx", idx}}, R"cc(
+      const $pb$::EnumDescriptor* $nonnull$ $Msg_Enum$_descriptor() {
+        $pbi$::AssignDescriptors(&$desc_table$);
+        return $file_level_enum_descriptors$[$idx$];
+      }
+    )cc");
+  }
+
+  // Always generate the data array, even on the simple cases because someone
+  // might be using it for TDP entries. If it is not used in the end, the linker
+  // will drop it.
+  p->Emit({{"encoded",
+            [&] {
+              for (uint32_t n :
+                   google::protobuf::internal::GenerateEnumData(sorted_unique_values_)) {
+                p->Emit({{"n", n}}, "$n$u, ");
+              }
+            }}},
+          R"cc(
+            PROTOBUF_CONSTINIT const uint32_t $Msg_Enum$_internal_data_[] = {
+                $encoded$};
+          )cc");
 
   if (!has_reflection_) {
     // In lite mode (where descriptors are unavailable), we generate separate
@@ -506,16 +526,15 @@ void EnumGenerator::GenerateMethods(int idx, io::Printer* p) {
              }},
         },
         R"cc(
-          static ::$proto_ns$::internal::ExplicitlyConstructed<std::string>
+          static $pbi$::ExplicitlyConstructed<std::string>
               $Msg_Enum$_strings[$num_unique$] = {};
 
           static const char $Msg_Enum$_names[] = {
               $names$,
           };
 
-          static const ::$proto_ns$::internal::EnumEntry $Msg_Enum$_entries[] =
-              {
-                  $entries$,
+          static const $pbi$::EnumEntry $Msg_Enum$_entries[] = {
+              $entries$,
           };
 
           static const int $Msg_Enum$_entries_by_number[] = {
@@ -523,22 +542,20 @@ void EnumGenerator::GenerateMethods(int idx, io::Printer* p) {
           };
 
           $return_type$ $Msg_Enum$_Name($Msg_Enum$ value) {
-            static const bool kDummy =
-                ::$proto_ns$::internal::InitializeEnumStrings(
-                    $Msg_Enum$_entries, $Msg_Enum$_entries_by_number,
-                    $num_unique$, $Msg_Enum$_strings);
+            static const bool kDummy = $pbi$::InitializeEnumStrings(
+                $Msg_Enum$_entries, $Msg_Enum$_entries_by_number, $num_unique$,
+                $Msg_Enum$_strings);
             (void)kDummy;
 
-            int idx = ::$proto_ns$::internal::LookUpEnumName(
-                $Msg_Enum$_entries, $Msg_Enum$_entries_by_number, $num_unique$,
-                value);
-            return idx == -1 ? ::$proto_ns$::internal::GetEmptyString()
-                             : $Msg_Enum$_strings[idx].get();
+            int idx = $pbi$::LookUpEnumName($Msg_Enum$_entries,
+                                            $Msg_Enum$_entries_by_number,
+                                            $num_unique$, value);
+            return idx == -1 ? $pbi$::GetEmptyString() : $Msg_Enum$_strings[idx].get();
           }
 
-          bool $Msg_Enum$_Parse(absl::string_view name, $Msg_Enum$* value) {
+          bool $Msg_Enum$_Parse(absl::string_view name, $Msg_Enum$* $nonnull$ value) {
             int int_value;
-            bool success = ::$proto_ns$::internal::LookUpEnumValue(
+            bool success = $pbi$::LookUpEnumValue(
                 $Msg_Enum$_entries, $num_declared$, name, &int_value);
             if (success) {
               *value = static_cast<$Msg_Enum$>(int_value);
@@ -547,45 +564,6 @@ void EnumGenerator::GenerateMethods(int idx, io::Printer* p) {
           }
         )cc");
   }
-
-  if (enum_->containing_type() != nullptr) {
-    // Before C++17, we must define the static constants which were
-    // declared in the header, to give the linker a place to put them.
-    // But MSVC++ pre-2015 and post-2017 (version 15.5+) insists that we not.
-    p->Emit(
-        {
-            {"Msg_", ClassName(enum_->containing_type(), false)},
-            {"constexpr_storage",
-             [&] {
-               for (int i = 0; i < enum_->value_count(); i++) {
-                 p->Emit({{"VALUE", EnumValueName(enum_->value(i))}},
-                         R"cc(
-                           constexpr $Msg_Enum$ $Msg_$::$VALUE$;
-                         )cc");
-               }
-             }},
-            {"array_size",
-             [&] {
-               if (generate_array_size_) {
-                 p->Emit(R"cc(
-                   constexpr int $Msg_$::$Enum$_ARRAYSIZE;
-                 )cc");
-               }
-             }},
-        },
-        R"(
-          #if (__cplusplus < 201703) && \
-            (!defined(_MSC_VER) || (_MSC_VER >= 1900 && _MSC_VER < 1912))
-          
-          $constexpr_storage$;
-          constexpr $Msg_Enum$ $Msg_$::$Enum$_MIN;
-          constexpr $Msg_Enum$ $Msg_$::$Enum$_MAX;
-          $array_size$;
-
-          #endif  // (__cplusplus < 201703) &&
-                  // (!defined(_MSC_VER) || (_MSC_VER >= 1900 && _MSC_VER < 1912))
-        )");
-  }
 }
 }  // namespace cpp
 }  // namespace compiler
