@@ -1,32 +1,9 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 // Author: kenton@google.com (Kenton Varda)
 //  Based on original Protocol Buffers design by
@@ -110,11 +87,9 @@ std::vector<Sub> Vars(const FieldDescriptor* field, const Options& options) {
 
 class SingularPrimitive final : public FieldGeneratorBase {
  public:
-  SingularPrimitive(const FieldDescriptor* field, const Options& opts)
-      : FieldGeneratorBase(field, opts),
-        field_(field),
-        opts_(&opts),
-        is_oneof_(field_->real_containing_oneof() != nullptr) {}
+  SingularPrimitive(const FieldDescriptor* field, const Options& opts,
+                    MessageSCCAnalyzer* scc)
+      : FieldGeneratorBase(field, opts, scc), opts_(&opts) {}
   ~SingularPrimitive() override = default;
 
   std::vector<Sub> MakeVars() const override { return Vars(field_, *opts_); }
@@ -133,12 +108,12 @@ class SingularPrimitive final : public FieldGeneratorBase {
 
   void GenerateMergingCode(io::Printer* p) const override {
     p->Emit(R"cc(
-      _this->_internal_set_$name$(from._internal_$name$());
+      _this->$field_$ = from.$field_$;
     )cc");
   }
 
   void GenerateSwappingCode(io::Printer* p) const override {
-    if (is_oneof_) {
+    if (is_oneof()) {
       // Don't print any swapping code. Swapping the union will swap this field.
       return;
     }
@@ -150,7 +125,7 @@ class SingularPrimitive final : public FieldGeneratorBase {
   }
 
   void GenerateConstructorCode(io::Printer* p) const override {
-    if (!is_oneof_) {
+    if (!is_oneof()) {
       return;
     }
 
@@ -189,9 +164,7 @@ class SingularPrimitive final : public FieldGeneratorBase {
   void GenerateByteSize(io::Printer* p) const override;
 
  private:
-  const FieldDescriptor* field_;
   const Options* opts_;
-  bool is_oneof_;
 };
 
 void SingularPrimitive::GenerateAccessorDeclarations(io::Printer* p) const {
@@ -214,43 +187,49 @@ void SingularPrimitive::GenerateInlineAccessorDefinitions(
     io::Printer* p) const {
   p->Emit(R"cc(
     inline $Type$ $Msg$::$name$() const {
+      $WeakDescriptorSelfPin$;
       $annotate_get$;
       // @@protoc_insertion_point(field_get:$pkg.Msg.field$)
-      return _internal_$name$();
-    }
-    inline void $Msg$::set_$name$($Type$ value) {
-      $PrepareSplitMessageForWrite$;
-      _internal_set_$name$(value);
-      $annotate_set$;
-      // @@protoc_insertion_point(field_set:$pkg.Msg.field$)
+      return _internal_$name_internal$();
     }
   )cc");
 
-  if (is_oneof_) {
+  if (is_oneof()) {
     p->Emit(R"cc(
-      inline $Type$ $Msg$::_internal_$name$() const {
+      inline void $Msg$::set_$name$($Type$ value) {
+        $WeakDescriptorSelfPin$;
+        $PrepareSplitMessageForWrite$;
+        if ($not_has_field$) {
+          clear_$oneof_name$();
+          set_has_$name_internal$();
+        }
+        $field_$ = value;
+        $annotate_set$;
+        // @@protoc_insertion_point(field_set:$pkg.Msg.field$)
+      }
+      inline $Type$ $Msg$::_internal_$name_internal$() const {
         if ($has_field$) {
           return $field_$;
         }
         return $kDefault$;
       }
-      inline void $Msg$::_internal_set_$name$($Type$ value) {
-        if ($not_has_field$) {
-          clear_$oneof_name$();
-          set_has_$name$();
-        }
-        $field_$ = value;
-      }
     )cc");
   } else {
     p->Emit(R"cc(
-      inline $Type$ $Msg$::_internal_$name$() const {
+      inline void $Msg$::set_$name$($Type$ value) {
+        $WeakDescriptorSelfPin$;
+        $PrepareSplitMessageForWrite$;
+        _internal_set_$name_internal$(value);
+        $set_hasbit$;
+        $annotate_set$;
+        // @@protoc_insertion_point(field_set:$pkg.Msg.field$)
+      }
+      inline $Type$ $Msg$::_internal_$name_internal$() const {
         $TsanDetectConcurrentRead$;
         return $field_$;
       }
-      inline void $Msg$::_internal_set_$name$($Type$ value) {
+      inline void $Msg$::_internal_set_$name_internal$($Type$ value) {
         $TsanDetectConcurrentMutation$;
-        $set_hasbit$;
         $field_$ = value;
       }
     )cc");
@@ -259,23 +238,23 @@ void SingularPrimitive::GenerateInlineAccessorDefinitions(
 
 void SingularPrimitive::GenerateSerializeWithCachedSizesToArray(
     io::Printer* p) const {
-  if ((descriptor_->number() < 16) &&
-      (descriptor_->type() == FieldDescriptor::TYPE_INT32 ||
-       descriptor_->type() == FieldDescriptor::TYPE_INT64 ||
-       descriptor_->type() == FieldDescriptor::TYPE_ENUM)) {
+  if ((field_->number() < 16) &&
+      (field_->type() == FieldDescriptor::TYPE_INT32 ||
+       field_->type() == FieldDescriptor::TYPE_INT64 ||
+       field_->type() == FieldDescriptor::TYPE_ENUM)) {
     // Call special non-inlined routine with tag number hardcoded as a
     // template parameter that handles the EnsureSpace and the writing
     // of the tag+value to the array
     p->Emit(R"cc(
-      target = ::$proto_ns$::internal::WireFormatLite::
-          Write$declared_type$ToArrayWithField<$number$>(
-              stream, this->_internal_$name$(), target);
+      target =
+          $pbi$::WireFormatLite::Write$declared_type$ToArrayWithField<$number$>(
+              stream, this_._internal_$name$(), target);
     )cc");
   } else {
     p->Emit(R"cc(
       target = stream->EnsureSpace(target);
       target = ::_pbi::WireFormatLite::Write$DeclaredType$ToArray(
-          $number$, this->_internal_$name$(), target);
+          $number$, this_._internal_$name$(), target);
     )cc");
   }
 }
@@ -296,53 +275,79 @@ void SingularPrimitive::GenerateByteSize(io::Printer* p) const {
   if (tag_size == 1) {
     p->Emit(R"cc(
       total_size += ::_pbi::WireFormatLite::$DeclaredType$SizePlusOne(
-          this->_internal_$name$());
+          this_._internal_$name$());
     )cc");
     return;
   }
 
   p->Emit(R"cc(
     total_size += $kTagBytes$ + ::_pbi::WireFormatLite::$DeclaredType$Size(
-                                    this->_internal_$name$());
+                                    this_._internal_$name$());
   )cc");
 }
 
 class RepeatedPrimitive final : public FieldGeneratorBase {
  public:
-  RepeatedPrimitive(const FieldDescriptor* field, const Options& opts)
-      : FieldGeneratorBase(field, opts), field_(field), opts_(&opts) {}
+  RepeatedPrimitive(const FieldDescriptor* field, const Options& opts,
+                    MessageSCCAnalyzer* scc)
+      : FieldGeneratorBase(field, opts, scc), opts_(&opts) {}
   ~RepeatedPrimitive() override = default;
 
   std::vector<Sub> MakeVars() const override { return Vars(field_, *opts_); }
 
   void GenerateClearingCode(io::Printer* p) const override {
-    p->Emit(R"cc(
-      _internal_mutable_$name$()->Clear();
-    )cc");
+    if (should_split()) {
+      p->Emit("$field_$.ClearIfNotDefault();\n");
+    } else {
+      p->Emit("$field_$.Clear();\n");
+    }
   }
 
   void GenerateMergingCode(io::Printer* p) const override {
-    p->Emit(R"cc(
-      _this->_internal_mutable_$name$()->MergeFrom(from._internal_$name$());
-    )cc");
+    // TODO: experiment with simplifying this to be
+    // `if (!from.empty()) { body(); }` for both split and non-split cases.
+    auto body = [&] {
+      p->Emit(R"cc(
+        _this->_internal_mutable_$name$()->MergeFrom(from._internal_$name$());
+      )cc");
+    };
+    if (!should_split()) {
+      body();
+    } else {
+      p->Emit({{"body", body}}, R"cc(
+        if (!from.$field_$.IsDefault()) {
+          $body$;
+        }
+      )cc");
+    }
   }
 
   void GenerateSwappingCode(io::Printer* p) const override {
-    ABSL_CHECK(!ShouldSplit(descriptor_, options_));
+    ABSL_CHECK(!should_split());
     p->Emit(R"cc(
       $field_$.InternalSwap(&other->$field_$);
     )cc");
   }
 
   void GenerateDestructorCode(io::Printer* p) const override {
-    p->Emit(R"cc(
-      $field_$.~RepeatedField();
-    )cc");
+    if (should_split()) {
+      p->Emit(R"cc(
+        this_.$field_$.DeleteIfNotDefault();
+      )cc");
+    }
   }
 
   void GenerateConstructorCode(io::Printer* p) const override {}
 
-  void GenerateCopyConstructorCode(io::Printer* p) const override {}
+  void GenerateCopyConstructorCode(io::Printer* p) const override {
+    if (should_split()) {
+      p->Emit(R"cc(
+        if (!from._internal_$name$().empty()) {
+          _internal_mutable_$name$()->MergeFrom(from._internal_$name$());
+        }
+      )cc");
+    }
+  }
 
   void GenerateConstexprAggregateInitializer(io::Printer* p) const override {
     p->Emit(R"cc(
@@ -352,7 +357,7 @@ class RepeatedPrimitive final : public FieldGeneratorBase {
   }
 
   void GenerateAggregateInitializer(io::Printer* p) const override {
-    ABSL_CHECK(!ShouldSplit(descriptor_, options_));
+    ABSL_CHECK(!should_split());
     p->Emit(R"cc(
       decltype($field_$){arena},
     )cc");
@@ -360,13 +365,38 @@ class RepeatedPrimitive final : public FieldGeneratorBase {
   }
 
   void GenerateCopyAggregateInitializer(io::Printer* p) const override {
-    ABSL_CHECK(!ShouldSplit(descriptor_, options_));
+    ABSL_CHECK(!should_split());
     p->Emit(R"cc(
       decltype($field_$){from.$field_$},
     )cc");
     GenerateCacheSizeInitializer(p);
   }
 
+  void GenerateMemberConstexprConstructor(io::Printer* p) const override {
+    p->Emit("$name$_{}");
+    if (HasCachedSize()) {
+      p->Emit(",\n_$name$_cached_byte_size_{0}");
+    }
+  }
+
+  void GenerateMemberConstructor(io::Printer* p) const override {
+    p->Emit("$name$_{visibility, arena}");
+    if (HasCachedSize()) {
+      p->Emit(",\n_$name$_cached_byte_size_{0}");
+    }
+  }
+
+  void GenerateMemberCopyConstructor(io::Printer* p) const override {
+    p->Emit("$name$_{visibility, arena, from.$name$_}");
+    if (HasCachedSize()) {
+      p->Emit(",\n_$name$_cached_byte_size_{0}");
+    }
+  }
+
+  void GenerateOneofCopyConstruct(io::Printer* p) const override {
+    ABSL_LOG(FATAL) << "Not supported";
+  }
+
   void GeneratePrivateMembers(io::Printer* p) const override;
   void GenerateAccessorDeclarations(io::Printer* p) const override;
   void GenerateInlineAccessorDefinitions(io::Printer* p) const override;
@@ -377,7 +407,8 @@ class RepeatedPrimitive final : public FieldGeneratorBase {
   bool HasCachedSize() const {
     bool is_packed_varint =
         field_->is_packed() && !FixedSize(field_->type()).has_value();
-    return is_packed_varint && HasGeneratedMethods(field_->file(), *opts_);
+    return is_packed_varint && HasGeneratedMethods(field_->file(), *opts_) &&
+           !should_split();
   }
 
   void GenerateCacheSizeInitializer(io::Printer* p) const {
@@ -389,19 +420,24 @@ class RepeatedPrimitive final : public FieldGeneratorBase {
     )cc");
   }
 
-  const FieldDescriptor* field_;
   const Options* opts_;
 };
 
 void RepeatedPrimitive::GeneratePrivateMembers(io::Printer* p) const {
-  p->Emit(R"cc(
-    $pb$::RepeatedField<$Type$> $name$_;
-  )cc");
+  if (should_split()) {
+    p->Emit(R"cc(
+      $pbi$::RawPtr<$pb$::RepeatedField<$Type$>> $name$_;
+    )cc");
+  } else {
+    p->Emit(R"cc(
+      $pb$::RepeatedField<$Type$> $name$_;
+    )cc");
+  }
 
   if (HasCachedSize()) {
     p->Emit({{"_cached_size_", MakeVarintCachedSizeName(field_)}},
             R"cc(
-              mutable $pbi$::CachedSize $_cached_size_$;
+              $pbi$::CachedSize $_cached_size_$;
             )cc");
   }
 }
@@ -418,11 +454,11 @@ void RepeatedPrimitive::GenerateAccessorDeclarations(io::Printer* p) const {
     $DEPRECATED$ void $set_name$(int index, $Type$ value);
     $DEPRECATED$ void $add_name$($Type$ value);
     $DEPRECATED$ const $pb$::RepeatedField<$Type$>& $name$() const;
-    $DEPRECATED$ $pb$::RepeatedField<$Type$>* $mutable_name$();
+    $DEPRECATED$ $pb$::RepeatedField<$Type$>* $nonnull$ $mutable_name$();
 
     private:
     const $pb$::RepeatedField<$Type$>& $_internal_name$() const;
-    $pb$::RepeatedField<$Type$>* $_internal_mutable_name$();
+    $pb$::RepeatedField<$Type$>* $nonnull$ $_internal_mutable_name$();
 
     public:
   )cc");
@@ -432,52 +468,90 @@ void RepeatedPrimitive::GenerateInlineAccessorDefinitions(
     io::Printer* p) const {
   p->Emit(R"cc(
     inline $Type$ $Msg$::$name$(int index) const {
+      $WeakDescriptorSelfPin$;
       $annotate_get$;
       // @@protoc_insertion_point(field_get:$pkg.Msg.field$)
-      return _internal_$name$().Get(index);
+      return _internal_$name_internal$().Get(index);
     }
+  )cc");
+  p->Emit(R"cc(
     inline void $Msg$::set_$name$(int index, $Type$ value) {
+      $WeakDescriptorSelfPin$;
       $annotate_set$;
-      _internal_mutable_$name$()->Set(index, value);
+      _internal_mutable_$name_internal$()->Set(index, value);
       // @@protoc_insertion_point(field_set:$pkg.Msg.field$)
     }
+  )cc");
+  p->Emit(R"cc(
     inline void $Msg$::add_$name$($Type$ value) {
+      $WeakDescriptorSelfPin$;
       $TsanDetectConcurrentMutation$;
-      _internal_mutable_$name$()->Add(value);
+      _internal_mutable_$name_internal$()->Add(value);
       $annotate_add$;
       // @@protoc_insertion_point(field_add:$pkg.Msg.field$)
     }
-    inline const $pb$::RepeatedField<$Type$>& $Msg$::$name$() const {
+  )cc");
+  p->Emit(R"cc(
+    inline const $pb$::RepeatedField<$Type$>& $Msg$::$name$() const
+        ABSL_ATTRIBUTE_LIFETIME_BOUND {
+      $WeakDescriptorSelfPin$;
       $annotate_list$;
       // @@protoc_insertion_point(field_list:$pkg.Msg.field$)
-      return _internal_$name$();
+      return _internal_$name_internal$();
     }
-    inline $pb$::RepeatedField<$Type$>* $Msg$::mutable_$name$() {
+  )cc");
+  p->Emit(R"cc(
+    inline $pb$::RepeatedField<$Type$>* $nonnull$ $Msg$::mutable_$name$()
+        ABSL_ATTRIBUTE_LIFETIME_BOUND {
+      $WeakDescriptorSelfPin$;
       $annotate_mutable_list$;
       // @@protoc_insertion_point(field_mutable_list:$pkg.Msg.field$)
       $TsanDetectConcurrentMutation$;
-      return _internal_mutable_$name$();
-    }
-
-    inline const $pb$::RepeatedField<$Type$>& $Msg$::_internal_$name$() const {
-      $TsanDetectConcurrentRead$;
-      return $field_$;
-    }
-    inline $pb$::RepeatedField<$Type$>* $Msg$::_internal_mutable_$name$() {
-      $TsanDetectConcurrentRead$;
-      return &$field_$;
+      return _internal_mutable_$name_internal$();
     }
   )cc");
+
+  if (should_split()) {
+    p->Emit(R"cc(
+      inline const $pb$::RepeatedField<$Type$>&
+      $Msg$::_internal_$name_internal$() const {
+        $TsanDetectConcurrentRead$;
+        return *$field_$;
+      }
+      inline $pb$::RepeatedField<$Type$>* $nonnull$
+      $Msg$::_internal_mutable_$name_internal$() {
+        $TsanDetectConcurrentRead$;
+        $PrepareSplitMessageForWrite$;
+        if ($field_$.IsDefault()) {
+          $field_$.Set($pb$::Arena::Create<$pb$::RepeatedField<$Type$>>(GetArena()));
+        }
+        return $field_$.Get();
+      }
+    )cc");
+  } else {
+    p->Emit(R"cc(
+      inline const $pb$::RepeatedField<$Type$>&
+      $Msg$::_internal_$name_internal$() const {
+        $TsanDetectConcurrentRead$;
+        return $field_$;
+      }
+      inline $pb$::RepeatedField<$Type$>* $nonnull$
+      $Msg$::_internal_mutable_$name_internal$() {
+        $TsanDetectConcurrentRead$;
+        return &$field_$;
+      }
+    )cc");
+  }
 }
 
 void RepeatedPrimitive::GenerateSerializeWithCachedSizesToArray(
     io::Printer* p) const {
   if (!field_->is_packed()) {
     p->Emit(R"cc(
-      for (int i = 0, n = this->_internal_$name$_size(); i < n; ++i) {
+      for (int i = 0, n = this_._internal_$name$_size(); i < n; ++i) {
         target = stream->EnsureSpace(target);
         target = ::_pbi::WireFormatLite::Write$DeclaredType$ToArray(
-            $number$, this->_internal_$name$().Get(i), target);
+            $number$, this_._internal_$name$().Get(i), target);
       }
     )cc");
     return;
@@ -485,90 +559,102 @@ void RepeatedPrimitive::GenerateSerializeWithCachedSizesToArray(
 
   if (FixedSize(field_->type()).has_value()) {
     p->Emit(R"cc(
-      if (this->_internal_$name$_size() > 0) {
-        target = stream->WriteFixedPacked($number$, _internal_$name$(), target);
+      if (this_._internal_$name$_size() > 0) {
+        target = stream->WriteFixedPacked($number$, this_._internal_$name$(), target);
       }
     )cc");
     return;
   }
 
-  p->Emit(R"cc(
-    {
-      int byte_size = $_field_cached_byte_size_$.Get();
-      if (byte_size > 0) {
-        target = stream->Write$DeclaredType$Packed($number$, _internal_$name$(),
-                                                   byte_size, target);
-      }
-    }
-  )cc");
-}
-
-void RepeatedPrimitive::GenerateByteSize(io::Printer* p) const {
   p->Emit(
       {
-          Sub{"data_size",
-              [&] {
-                auto fixed_size = FixedSize(descriptor_->type());
-                if (fixed_size.has_value()) {
-                  p->Emit({{"kFixed", *fixed_size}}, R"cc(
-                    std::size_t{$kFixed$} *
-                        ::_pbi::FromIntSize(this->_internal_$name$_size())
-                  )cc");
-                } else {
-                  p->Emit(R"cc(
-                    ::_pbi::WireFormatLite::$DeclaredType$Size(
-                        this->_internal_$name$())
-                  )cc");
-                }
-              }}  // Here and below, we need to disable the default ;-chomping
-                  // that closure substitutions do.
-              .WithSuffix(""),
-          {"maybe_cache_data_size",
+          {"byte_size",
            [&] {
-             if (!HasCachedSize()) return;
-             p->Emit(R"cc(
-               $_field_cached_byte_size_$.Set(::_pbi::ToCachedSize(data_size));
-             )cc");
+             if (HasCachedSize()) {
+               p->Emit(R"cc(this_.$_field_cached_byte_size_$.Get();)cc");
+             } else {
+               p->Emit(R"cc(
+                 ::_pbi::WireFormatLite::$DeclaredType$Size(
+                     this_._internal_$name$());
+               )cc");
+             }
            }},
-          Sub{"tag_size",
-              [&] {
-                if (field_->is_packed()) {
-                  p->Emit(R"cc(
-                    data_size == 0
-                        ? 0
-                        : $kTagBytes$ + ::_pbi::WireFormatLite::Int32Size(
-                                            static_cast<int32_t>(data_size))
-                  )cc");
-                } else {
-                  p->Emit(R"cc(
-                    std::size_t{$kTagBytes$} *
-                        ::_pbi::FromIntSize(this->_internal_$name$_size());
-                  )cc");
-                }
-              }}
-              .WithSuffix(""),
       },
       R"cc(
         {
-          std::size_t data_size = $data_size$;
-          $maybe_cache_data_size$;
-          std::size_t tag_size = $tag_size$;
-          total_size += tag_size + data_size;
+          int byte_size = $byte_size$;
+          if (byte_size > 0) {
+            target = stream->Write$DeclaredType$Packed(
+                $number$, this_._internal_$name$(), byte_size, target);
+          }
         }
       )cc");
 }
+
+void RepeatedPrimitive::GenerateByteSize(io::Printer* p) const {
+  if (HasCachedSize()) {
+    ABSL_CHECK(field_->is_packed());
+    p->Emit(
+        R"cc(
+          total_size +=
+              ::_pbi::WireFormatLite::$DeclaredType$SizeWithPackedTagSize(
+                  this_._internal_$name$(), $kTagBytes$,
+                  this_.$_field_cached_byte_size_$);
+        )cc");
+    return;
+  }
+  p->Emit(
+      {
+          {"data_size",
+           [&] {
+             auto fixed_size = FixedSize(field_->type());
+             if (fixed_size.has_value()) {
+               p->Emit({{"kFixed", *fixed_size}}, R"cc(
+                 std::size_t{$kFixed$} *
+                     ::_pbi::FromIntSize(this_._internal_$name$_size());
+               )cc");
+             } else {
+               p->Emit(R"cc(
+                 ::_pbi::WireFormatLite::$DeclaredType$Size(
+                     this_._internal_$name$());
+               )cc");
+             }
+           }},
+          {"tag_size",
+           [&] {
+             if (field_->is_packed()) {
+               p->Emit(R"cc(
+                 data_size == 0
+                     ? 0
+                     : $kTagBytes$ + ::_pbi::WireFormatLite::Int32Size(
+                                         static_cast<::int32_t>(data_size));
+               )cc");
+             } else {
+               p->Emit(R"cc(
+                 std::size_t{$kTagBytes$} *
+                     ::_pbi::FromIntSize(this_._internal_$name$_size());
+               )cc");
+             }
+           }},
+      },
+      R"cc(
+        std::size_t data_size = $data_size$;
+        std::size_t tag_size = $tag_size$;
+        total_size += tag_size + data_size;
+      )cc");
+}
 }  // namespace
 
 std::unique_ptr<FieldGeneratorBase> MakeSinguarPrimitiveGenerator(
     const FieldDescriptor* desc, const Options& options,
     MessageSCCAnalyzer* scc) {
-  return absl::make_unique<SingularPrimitive>(desc, options);
+  return absl::make_unique<SingularPrimitive>(desc, options, scc);
 }
 
 std::unique_ptr<FieldGeneratorBase> MakeRepeatedPrimitiveGenerator(
     const FieldDescriptor* desc, const Options& options,
     MessageSCCAnalyzer* scc) {
-  return absl::make_unique<RepeatedPrimitive>(desc, options);
+  return absl::make_unique<RepeatedPrimitive>(desc, options, scc);
 }
 
 }  // namespace cpp
