@@ -52,7 +52,6 @@ void SetPrimitiveVariables(
   JavaType javaType = GetJavaType(descriptor);
   (*variables)["type"] = std::string(PrimitiveTypeName(javaType));
   (*variables)["boxed_type"] = std::string(BoxedPrimitiveTypeName(javaType));
-  (*variables)["kt_type"] = std::string(KotlinTypeName(javaType));
   variables->insert({"field_type", (*variables)["type"]});
   (*variables)["default"] =
       ImmutableDefaultValue(descriptor, name_resolver, context->options());
@@ -121,11 +120,6 @@ void SetPrimitiveVariables(
   // by the proto compiler
   (*variables)["deprecation"] =
       descriptor->options().deprecated() ? "@java.lang.Deprecated " : "";
-  (*variables)["kt_deprecation"] =
-      descriptor->options().deprecated()
-          ? absl::StrCat("@kotlin.Deprecated(message = \"Field ", name,
-                         " is deprecated\") ")
-          : "";
   int fixed_size = FixedSize(GetType(descriptor));
   if (fixed_size != -1) {
     (*variables)["fixed_size"] = absl::StrCat(fixed_size);
@@ -133,8 +127,6 @@ void SetPrimitiveVariables(
 
   if (HasHasbit(descriptor)) {
     // For singular messages and builders, one bit is used for the hasField bit.
-    (*variables)["get_has_field_bit_message"] = GenerateGetBit(messageBitIndex);
-
     // Note that these have a trailing ";".
     (*variables)["set_has_field_bit_message"] =
         absl::StrCat(GenerateSetBit(messageBitIndex), ";");
@@ -237,7 +229,7 @@ void ImmutablePrimitiveFieldLiteGenerator::GenerateMembers(
         variables_,
         "@java.lang.Override\n"
         "$deprecation$public boolean ${$has$capitalized_name$$}$() {\n"
-        "  return $get_has_field_bit_message$;\n"
+        "  return $is_field_present_message$;\n"
         "}\n");
     printer->Annotate("{", "}", descriptor_);
   }
@@ -326,63 +318,6 @@ void ImmutablePrimitiveFieldLiteGenerator::GenerateBuilderMembers(
   printer->Annotate("{", "}", descriptor_, Semantic::kSet);
 }
 
-void ImmutablePrimitiveFieldLiteGenerator::GenerateKotlinDslMembers(
-    io::Printer* printer) const {
-  auto vars = printer->WithVars(variables_);
-  JvmNameContext name_ctx = {context_->options(), printer};
-  WriteFieldDocComment(printer, descriptor_, context_->options());
-  if (descriptor_->name() == "is_initialized") {
-    printer->Emit(
-        {
-            {"jvm_name_get",
-             [&] { JvmName("${$get$kt_capitalized_name$$}$", name_ctx); }},
-            {"jvm_name_set",
-             [&] { JvmName("${$set$kt_capitalized_name$$}$", name_ctx); }},
-        },
-        "// TODO: b/336400327 - remove this hack; we should access properties\n"
-        "$kt_deprecation$public var $kt_name$: $kt_type$\n"
-        "  $jvm_name_get$"
-        "  get() = $kt_dsl_builder$.get${$$kt_capitalized_name$$}$()\n"
-        "  $jvm_name_set$"
-        "  set(value) {\n"
-        "    $kt_dsl_builder$.${$set$kt_capitalized_name$$}$(value)\n"
-        "  }\n");
-  } else {
-    printer->Emit(
-        {
-            {"jvm_name_get",
-             [&] { JvmName("${$get$kt_capitalized_name$$}$", name_ctx); }},
-            {"jvm_name_set",
-             [&] { JvmName("${$set$kt_capitalized_name$$}$", name_ctx); }},
-        },
-        "$kt_deprecation$public var $kt_name$: $kt_type$\n"
-        "  $jvm_name_get$"
-        "  get() = $kt_dsl_builder$.${$$kt_safe_name$$}$\n"
-        "  $jvm_name_set$"
-        "  set(value) {\n"
-        "    $kt_dsl_builder$.${$$kt_safe_name$$}$ = value\n"
-        "  }\n");
-  }
-
-  WriteFieldAccessorDocComment(printer, descriptor_, CLEARER,
-                               context_->options(),
-                               /* builder */ false, /* kdoc */ true);
-  printer->Print(
-      "public fun ${$clear$kt_capitalized_name$$}$() {\n"
-      "  $kt_dsl_builder$.${$clear$capitalized_name$$}$()\n"
-      "}\n");
-
-  if (descriptor_->has_presence()) {
-    WriteFieldAccessorDocComment(printer, descriptor_, HAZZER,
-                                 context_->options(),
-                                 /* builder */ false, /* kdoc */ true);
-    printer->Print(
-        "public fun ${$has$kt_capitalized_name$$}$(): kotlin.Boolean {\n"
-        "  return $kt_dsl_builder$.${$has$capitalized_name$$}$()\n"
-        "}\n");
-  }
-}
-
 void ImmutablePrimitiveFieldLiteGenerator::GenerateFieldInfo(
     io::Printer* printer, std::vector<uint16_t>* output) const {
   WriteIntToUtf16CharSequence(descriptor_->number(), output);
@@ -723,127 +658,6 @@ void RepeatedImmutablePrimitiveFieldLiteGenerator::GenerateBuilderMembers(
   printer->Annotate("{", "}", descriptor_, Semantic::kSet);
 }
 
-void RepeatedImmutablePrimitiveFieldLiteGenerator::GenerateKotlinDslMembers(
-    io::Printer* printer) const {
-  auto vars = printer->WithVars(variables_);
-  JvmNameContext name_ctx = {context_->options(), printer};
-  printer->Print(
-      "/**\n"
-      " * An uninstantiable, behaviorless type to represent the field in\n"
-      " * generics.\n"
-      " */\n"
-      "@kotlin.OptIn"
-      "(com.google.protobuf.kotlin.OnlyForUseByGeneratedProtoCode::class)\n"
-      "public class ${$$kt_capitalized_name$Proxy$}$ private constructor()"
-      " : com.google.protobuf.kotlin.DslProxy()\n");
-
-  WriteFieldDocComment(printer, descriptor_, context_->options());
-  printer->Print(
-      "$kt_deprecation$ public val $kt_name$: "
-      "com.google.protobuf.kotlin.DslList"
-      "<$kt_type$, ${$$kt_capitalized_name$Proxy$}$>\n"
-      "$  jvm_synthetic$"
-      "  get() = com.google.protobuf.kotlin.DslList(\n"
-      "    $kt_dsl_builder$.${$$kt_property_name$List$}$\n"
-      "  )\n");
-
-  WriteFieldAccessorDocComment(printer, descriptor_, LIST_ADDER,
-                               context_->options(),
-                               /* builder */ false, /* kdoc */ true);
-  printer->Emit(
-      {
-          {"jvm_name", [&] { JvmName("add$kt_capitalized_name$", name_ctx); }},
-      },
-      "$jvm_synthetic$"
-      "$jvm_name$"
-      "public fun com.google.protobuf.kotlin.DslList"
-      "<$kt_type$, ${$$kt_capitalized_name$Proxy$}$>."
-      "add(value: $kt_type$) {\n"
-      "  $kt_dsl_builder$.${$add$capitalized_name$$}$(value)\n"
-      "}");
-
-  WriteFieldAccessorDocComment(printer, descriptor_, LIST_ADDER,
-                               context_->options(),
-                               /* builder */ false, /* kdoc */ true);
-  printer->Emit(
-      {
-          {"jvm_name",
-           [&] { JvmName("plusAssign$kt_capitalized_name$", name_ctx); }},
-      },
-      "$jvm_synthetic$"
-      "$jvm_name$"
-      "@Suppress(\"NOTHING_TO_INLINE\")\n"
-      "public inline operator fun com.google.protobuf.kotlin.DslList"
-      "<$kt_type$, ${$$kt_capitalized_name$Proxy$}$>."
-      "plusAssign(value: $kt_type$) {\n"
-      "  add(value)\n"
-      "}");
-
-  WriteFieldAccessorDocComment(printer, descriptor_, LIST_MULTI_ADDER,
-                               context_->options(),
-                               /* builder */ false, /* kdoc */ true);
-  printer->Emit(
-      {
-          {"jvm_name",
-           [&] { JvmName("addAll$kt_capitalized_name$", name_ctx); }},
-      },
-      "$jvm_synthetic$"
-      "$jvm_name$"
-      "public fun com.google.protobuf.kotlin.DslList"
-      "<$kt_type$, ${$$kt_capitalized_name$Proxy$}$>."
-      "addAll(values: kotlin.collections.Iterable<$kt_type$>) {\n"
-      "  $kt_dsl_builder$.${$addAll$capitalized_name$$}$(values)\n"
-      "}");
-
-  WriteFieldAccessorDocComment(printer, descriptor_, LIST_MULTI_ADDER,
-                               context_->options(),
-                               /* builder */ false, /* kdoc */ true);
-  printer->Emit(
-      {
-          {"jvm_name",
-           [&] { JvmName("plusAssignAll$kt_capitalized_name$", name_ctx); }},
-      },
-      "$jvm_synthetic$"
-      "$jvm_name$"
-      "@Suppress(\"NOTHING_TO_INLINE\")\n"
-      "public inline operator fun com.google.protobuf.kotlin.DslList"
-      "<$kt_type$, ${$$kt_capitalized_name$Proxy$}$>."
-      "plusAssign(values: kotlin.collections.Iterable<$kt_type$>) {\n"
-      "  addAll(values)\n"
-      "}");
-
-  WriteFieldAccessorDocComment(printer, descriptor_, LIST_INDEXED_SETTER,
-                               context_->options(),
-                               /* builder */ false, /* kdoc */ true);
-  printer->Emit(
-      {
-          {"jvm_name", [&] { JvmName("set$kt_capitalized_name$", name_ctx); }},
-      },
-      "$jvm_synthetic$"
-      "$jvm_name$"
-      "public operator fun com.google.protobuf.kotlin.DslList"
-      "<$kt_type$, ${$$kt_capitalized_name$Proxy$}$>."
-      "set(index: kotlin.Int, value: $kt_type$) {\n"
-      "  $kt_dsl_builder$.${$set$capitalized_name$$}$(index, value)\n"
-      "}");
-
-  WriteFieldAccessorDocComment(printer, descriptor_, CLEARER,
-                               context_->options(),
-                               /* builder */ false, /* kdoc */ true);
-  printer->Emit(
-      {
-          {"jvm_name",
-           [&] { JvmName("clear$kt_capitalized_name$", name_ctx); }},
-      },
-      "$jvm_synthetic$"
-      "$jvm_name$"
-      "public fun com.google.protobuf.kotlin.DslList"
-      "<$kt_type$, ${$$kt_capitalized_name$Proxy$}$>."
-      "clear() {\n"
-      "  $kt_dsl_builder$.${$clear$capitalized_name$$}$()\n"
-      "}");
-}
-
 void RepeatedImmutablePrimitiveFieldLiteGenerator::GenerateFieldInfo(
     io::Printer* printer, std::vector<uint16_t>* output) const {
   WriteIntToUtf16CharSequence(descriptor_->number(), output);
