@@ -36,8 +36,9 @@
 #include "google/protobuf/map_test_util.h"
 #include "google/protobuf/map_unittest.pb.h"
 #include "google/protobuf/message.h"
+#include "google/protobuf/port.h"
 #include "google/protobuf/test_util.h"
-#include "google/protobuf/unittest.pb.h"
+#include "google/protobuf/text_format.h"
 #include "google/protobuf/unittest.pb.h"
 #include "google/protobuf/unittest_mset.pb.h"
 #include "google/protobuf/unittest_mset_wire_format.pb.h"
@@ -193,9 +194,9 @@ TEST(GeneratedMessageReflectionTest, GetStringViewWithExtensions) {
       descriptor_file->FindExtensionByName("optional_string_piece_extension");
   google::protobuf::FieldDescriptor const* cord_ext =
       descriptor_file->FindExtensionByName("optional_cord_extension");
-  message.SetExtension(protobuf_unittest::optional_string_extension, "foo");
-  message.SetExtension(protobuf_unittest::optional_string_piece_extension, "bar");
-  message.SetExtension(protobuf_unittest::optional_cord_extension, "baz");
+  message.SetExtension(proto2_unittest::optional_string_extension, "foo");
+  message.SetExtension(proto2_unittest::optional_string_piece_extension, "bar");
+  message.SetExtension(proto2_unittest::optional_cord_extension, "baz");
   const Reflection* reflection = message.GetReflection();
   Reflection::ScratchSpace scratch;
 
@@ -247,9 +248,9 @@ TEST(GeneratedMessageReflectionTest, GetRepeatedStringViewWithExtensions) {
       descriptor_file->FindExtensionByName("repeated_string_piece_extension");
   google::protobuf::FieldDescriptor const* cord_ext =
       descriptor_file->FindExtensionByName("repeated_cord_extension");
-  message.AddExtension(protobuf_unittest::repeated_string_extension, "foo");
-  message.AddExtension(protobuf_unittest::repeated_string_piece_extension, "bar");
-  message.AddExtension(protobuf_unittest::repeated_cord_extension, "baz");
+  message.AddExtension(proto2_unittest::repeated_string_extension, "foo");
+  message.AddExtension(proto2_unittest::repeated_string_piece_extension, "bar");
+  message.AddExtension(proto2_unittest::repeated_cord_extension, "baz");
   const Reflection* reflection = message.GetReflection();
   Reflection::ScratchSpace scratch;
 
@@ -735,20 +736,20 @@ TEST(GeneratedMessageReflectionTest, ReleaseLast) {
   message.Clear();
   TestUtil::SetAllFields(&message);
   ASSERT_EQ(2, message.repeated_foreign_message_size());
-  const protobuf_unittest::ForeignMessage* expected =
+  const proto2_unittest::ForeignMessage* expected =
       message.mutable_repeated_foreign_message(1);
   (void)expected;  // unused in somce configurations
   std::unique_ptr<Message> released(message.GetReflection()->ReleaseLast(
       &message, descriptor->FindFieldByName("repeated_foreign_message")));
-#ifndef PROTOBUF_FORCE_COPY_IN_RELEASE
-  EXPECT_EQ(expected, released.get());
-#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
+  if (!internal::DebugHardenForceCopyInRelease()) {
+    EXPECT_EQ(expected, released.get());
+  }
 }
 
 TEST(GeneratedMessageReflectionTest, ReleaseLastExtensions) {
-#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
-  GTEST_SKIP() << "Won't work with FORCE_COPY_IN_RELEASE.";
-#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
+  if (internal::DebugHardenForceCopyInRelease()) {
+    GTEST_SKIP() << "Won't work with FORCE_COPY_IN_RELEASE.";
+  }
 
   unittest::TestAllExtensions message;
   const Descriptor* descriptor = message.GetDescriptor();
@@ -765,7 +766,7 @@ TEST(GeneratedMessageReflectionTest, ReleaseLastExtensions) {
   TestUtil::SetAllExtensions(&message);
   ASSERT_EQ(
       2, message.ExtensionSize(unittest::repeated_foreign_message_extension));
-  const protobuf_unittest::ForeignMessage* expected =
+  const proto2_unittest::ForeignMessage* expected =
       message.MutableExtension(unittest::repeated_foreign_message_extension, 1);
   std::unique_ptr<Message> released(message.GetReflection()->ReleaseLast(
       &message, descriptor->file()->FindExtensionByName(
@@ -1198,9 +1199,9 @@ TEST(GeneratedMessageReflectionTest, SetAllocatedOneofMessageTest) {
   released = reflection->ReleaseMessage(
       &to_message, descriptor->FindFieldByName("foo_lazy_message"));
   EXPECT_TRUE(released != nullptr);
-#ifndef PROTOBUF_FORCE_COPY_IN_RELEASE
-  EXPECT_EQ(&sub_message, released);
-#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
+  if (!internal::DebugHardenForceCopyInRelease()) {
+    EXPECT_EQ(&sub_message, released);
+  }
   delete released;
 
   TestUtil::ReflectionTester::SetOneofViaReflection(&from_message2);
@@ -1218,9 +1219,9 @@ TEST(GeneratedMessageReflectionTest, SetAllocatedOneofMessageTest) {
   released = reflection->ReleaseMessage(
       &to_message, descriptor->FindFieldByName("foo_message"));
   EXPECT_TRUE(released != nullptr);
-#ifndef PROTOBUF_FORCE_COPY_IN_RELEASE
-  EXPECT_EQ(&sub_message2, released);
-#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
+  if (!internal::DebugHardenForceCopyInRelease()) {
+    EXPECT_EQ(&sub_message2, released);
+  }
   delete released;
 }
 
@@ -1341,9 +1342,9 @@ TEST(GeneratedMessageReflectionTest, ReleaseOneofMessageTest) {
       &message, descriptor->FindFieldByName("foo_lazy_message"));
 
   EXPECT_TRUE(released != nullptr);
-#ifndef PROTOBUF_FORCE_COPY_IN_RELEASE
-  EXPECT_EQ(&sub_message, released);
-#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
+  if (!internal::DebugHardenForceCopyInRelease()) {
+    EXPECT_EQ(&sub_message, released);
+  }
   delete released;
 
   released = reflection->ReleaseMessage(
@@ -1432,8 +1433,8 @@ TEST(GeneratedMessageReflectionTest, UsageErrors) {
                            descriptor->FindFieldByName("optional_int64")),
       "Protocol Buffer reflection usage error:\n"
       "  Method      : google::protobuf::Reflection::GetInt32\n"
-      "  Message type: protobuf_unittest\\.TestAllTypes\n"
-      "  Field       : protobuf_unittest\\.TestAllTypes\\.optional_int64\n"
+      "  Message type: proto2_unittest\\.TestAllTypes\n"
+      "  Field       : proto2_unittest\\.TestAllTypes\\.optional_int64\n"
       "  Problem     : Field is not the right type for this message:\n"
       "    Expected  : CPPTYPE_INT32\n"
       "    Field type: CPPTYPE_INT64");
@@ -1441,8 +1442,8 @@ TEST(GeneratedMessageReflectionTest, UsageErrors) {
                    message, descriptor->FindFieldByName("repeated_int32")),
                "Protocol Buffer reflection usage error:\n"
                "  Method      : google::protobuf::Reflection::GetInt32\n"
-               "  Message type: protobuf_unittest.TestAllTypes\n"
-               "  Field       : protobuf_unittest.TestAllTypes.repeated_int32\n"
+               "  Message type: proto2_unittest.TestAllTypes\n"
+               "  Field       : proto2_unittest.TestAllTypes.repeated_int32\n"
                "  Problem     : Field is repeated; the method requires a "
                "singular field.");
 #ifndef NDEBUG
@@ -1451,9 +1452,9 @@ TEST(GeneratedMessageReflectionTest, UsageErrors) {
                            descriptor->FindFieldByName("optional_int32")),
       "Protocol Buffer reflection usage error:\n"
       "  Method       : google::protobuf::Reflection::GetInt32\n"
-      "  Expected type: protobuf_unittest.TestAllTypes\n"
-      "  Actual type  : protobuf_unittest.ForeignMessage\n"
-      "  Field        : protobuf_unittest.TestAllTypes.optional_int32\n"
+      "  Expected type: proto2_unittest.TestAllTypes\n"
+      "  Actual type  : proto2_unittest.ForeignMessage\n"
+      "  Field        : proto2_unittest.TestAllTypes.optional_int32\n"
       "  Problem      : Message is not the right object for reflection");
 #endif
   EXPECT_DEATH(
@@ -1462,8 +1463,8 @@ TEST(GeneratedMessageReflectionTest, UsageErrors) {
           unittest::ForeignMessage::descriptor()->FindFieldByName("c")),
       "Protocol Buffer reflection usage error:\n"
       "  Method      : google::protobuf::Reflection::GetInt32\n"
-      "  Message type: protobuf_unittest.TestAllTypes\n"
-      "  Field       : protobuf_unittest.ForeignMessage.c\n"
+      "  Message type: proto2_unittest.TestAllTypes\n"
+      "  Field       : proto2_unittest.ForeignMessage.c\n"
       "  Problem     : Field does not match message type.");
   EXPECT_DEATH(
       reflection->HasField(
@@ -1471,8 +1472,8 @@ TEST(GeneratedMessageReflectionTest, UsageErrors) {
           unittest::ForeignMessage::descriptor()->FindFieldByName("c")),
       "Protocol Buffer reflection usage error:\n"
       "  Method      : google::protobuf::Reflection::HasField\n"
-      "  Message type: protobuf_unittest.TestAllTypes\n"
-      "  Field       : protobuf_unittest.ForeignMessage.c\n"
+      "  Message type: proto2_unittest.TestAllTypes\n"
+      "  Field       : proto2_unittest.ForeignMessage.c\n"
       "  Problem     : Field does not match message type.");
 }
 
@@ -1807,6 +1808,22 @@ TEST(GeneratedMessageReflection, SwapImplicitPresenceShouldWork) {
   EXPECT_EQ(lhs.child().optional_int32(), -1);
 }
 
+TEST(GeneratedMessageReflection, UnvalidatedStringsAreDowngradedToBytes) {
+  proto2_unittest::TestChildExtension parsed_msg;
+  google::protobuf::TextFormat::ParseFromString(
+      R"pb(
+        optional_extension <
+          [proto2_unittest.repeated_string_extension]: "foo"
+        >
+      )pb",
+      &parsed_msg);
+  proto2_unittest::TestChildExtension msg;
+  msg.mutable_optional_extension()->AddExtension(
+      proto2_unittest::repeated_string_extension, "bar");
+  parsed_msg.mutable_optional_extension()->Swap(
+      msg.mutable_optional_extension());
+}
+
 }  // namespace
 }  // namespace protobuf
 }  // namespace google
