@@ -1,36 +1,17 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 #ifndef GOOGLE_PROTOBUF_REFLECTION_TESTER_H__
 #define GOOGLE_PROTOBUF_REFLECTION_TESTER_H__
 
+#include <optional>
+
+#include "absl/strings/string_view.h"
+#include "google/protobuf/map_field.h"
 #include "google/protobuf/message.h"
 
 // Must be included last.
@@ -60,16 +41,37 @@ class MapReflectionTester {
   void ExpectClearViaReflection(const Message& message);
   void ExpectClearViaReflectionIterator(Message* message);
   void GetMapValueViaMapReflection(Message* message,
-                                   const std::string& field_name,
+                                   absl::string_view field_name,
                                    const MapKey& map_key, MapValueRef* map_val);
+  void DeleteMapValueViaMapReflection(Message* message,
+                                      absl::string_view field_name,
+                                      const MapKey& map_key);
   Message* GetMapEntryViaReflection(Message* message,
-                                    const std::string& field_name, int index);
-  MapIterator MapBegin(Message* message, const std::string& field_name);
-  MapIterator MapEnd(Message* message, const std::string& field_name);
-  int MapSize(const Message& message, const std::string& field_name);
+                                    absl::string_view field_name, int index);
+  MapIterator MapBegin(Message* message, absl::string_view field_name);
+  MapIterator MapEnd(Message* message, absl::string_view field_name);
+  int MapSize(const Message& message, absl::string_view field_name);
+
+  static std::optional<MapValueConstRef> LookupMapValue(
+      const Reflection& reflection, const Message& message,
+      const FieldDescriptor& descriptor, const MapKey& map_key) {
+    MapValueConstRef map_val_const;
+    if (reflection.LookupMapValue(message, &descriptor, map_key,
+                                  &map_val_const)) {
+      return map_val_const;
+    }
+    return std::nullopt;
+  }
+
+  static std::string long_string() {
+    return "This is a very long string that goes in the heap";
+  }
+  static std::string long_string_2() {
+    return "This is another very long string that goes in the heap";
+  }
 
  private:
-  const FieldDescriptor* F(const std::string& name);
+  const FieldDescriptor* F(absl::string_view name);
 
   const Descriptor* base_descriptor_;
 
