@@ -14,16 +14,17 @@
 
 #include "google/protobuf/unknown_field_set.h"
 
+#include <cstddef>
 #include <string>
 #include <vector>
 
 #include "google/protobuf/stubs/callback.h"
 #include "google/protobuf/stubs/common.h"
 #include <gmock/gmock.h>
-#include "google/protobuf/testing/googletest.h"
 #include <gtest/gtest.h>
 #include "absl/container/flat_hash_set.h"
 #include "absl/functional/bind_front.h"
+#include "absl/log/absl_check.h"
 #include "absl/strings/cord.h"
 #include "absl/synchronization/mutex.h"
 #include "absl/time/clock.h"
@@ -37,13 +38,28 @@
 #include "google/protobuf/unittest_lite.pb.h"
 #include "google/protobuf/wire_format.h"
 
-
 namespace google {
 namespace protobuf {
+namespace internal {
+struct UnknownFieldSetTestPeer {
+  static auto AddLengthDelimited(UnknownFieldSet& set, int number) {
+    return set.AddLengthDelimited(number);
+  }
+};
+}  // namespace internal
 
 using internal::WireFormat;
 using ::testing::ElementsAre;
 
+template <typename T>
+T UnknownToProto(const UnknownFieldSet& set) {
+  T message;
+  std::string serialized_message;
+  ABSL_CHECK(set.SerializeToString(&serialized_message));
+  ABSL_CHECK(message.ParseFromString(serialized_message));
+  return message;
+}
+
 class UnknownFieldSetTest : public testing::Test {
  protected:
   void SetUp() override {
@@ -176,6 +192,96 @@ TEST_F(UnknownFieldSetTest, Group) {
   EXPECT_EQ(all_fields_.optionalgroup().a(), nested_field.varint());
 }
 
+static void PopulateUFS(UnknownFieldSet& set) {
+  UnknownFieldSet* node = &set;
+  for (int i = 0; i < 3; ++i) {
+    node->AddVarint(1, 100);
+    const char* long_str = "This is a very long string, not sso";
+    node->AddLengthDelimited(2, long_str);
+    node->AddLengthDelimited(2, std::string(long_str));
+    node->AddLengthDelimited(2, absl::Cord(long_str));
+    *internal::UnknownFieldSetTestPeer::AddLengthDelimited(*node, 3) = long_str;
+    // Test some recursion too.
+    node = node->AddGroup(4);
+  }
+}
+
+TEST_F(UnknownFieldSetTest, ArenaSupportWorksWithMergeFrom) {
+  Arena arena;
+
+  for (bool lhs_arena : {false, true}) {
+    for (bool rhs_arena : {false, true}) {
+      UnknownFieldSet lhs_stack, rhs_stack;
+      auto& lhs =
+          lhs_arena ? *Arena::Create<UnknownFieldSet>(&arena) : lhs_stack;
+      auto& rhs =
+          rhs_arena ? *Arena::Create<UnknownFieldSet>(&arena) : rhs_stack;
+      PopulateUFS(rhs);
+      lhs.MergeFrom(rhs);
+    }
+  }
+}
+
+TEST_F(UnknownFieldSetTest, ArenaSupportWorksWithMergeAndDestroy) {
+  Arena arena;
+
+  for (bool lhs_arena : {false, true}) {
+    for (bool populate_lhs : {false, true}) {
+      for (bool rhs_arena : {false, true}) {
+        for (bool populate_rhs : {false, true}) {
+          UnknownFieldSet lhs_stack, rhs_stack;
+          auto& lhs =
+              lhs_arena ? *Arena::Create<UnknownFieldSet>(&arena) : lhs_stack;
+          auto& rhs =
+              rhs_arena ? *Arena::Create<UnknownFieldSet>(&arena) : rhs_stack;
+          if (populate_lhs) PopulateUFS(lhs);
+          if (populate_rhs) PopulateUFS(rhs);
+          lhs.MergeFromAndDestroy(&rhs);
+        }
+      }
+    }
+  }
+}
+
+TEST_F(UnknownFieldSetTest, ArenaSupportWorksWithSwap) {
+  Arena arena;
+
+  for (bool lhs_arena : {false, true}) {
+    for (bool rhs_arena : {false, true}) {
+      UnknownFieldSet lhs_stack, rhs_stack;
+      auto& lhs =
+          lhs_arena ? *Arena::Create<UnknownFieldSet>(&arena) : lhs_stack;
+      auto& rhs =
+          rhs_arena ? *Arena::Create<UnknownFieldSet>(&arena) : rhs_stack;
+      PopulateUFS(lhs);
+      lhs.Swap(&rhs);
+    }
+  }
+}
+
+TEST_F(UnknownFieldSetTest, ArenaSupportWorksWithClear) {
+  Arena arena;
+  auto* ufs = Arena::Create<UnknownFieldSet>(&arena);
+  PopulateUFS(*ufs);
+  // Clear should not try to delete memory from the arena.
+  ufs->Clear();
+}
+
+TEST_F(UnknownFieldSetTest, ArenaSupportWorksDelete) {
+  Arena arena;
+
+  auto* ufs = Arena::Create<UnknownFieldSet>(&arena);
+  PopulateUFS(*ufs);
+
+  while (ufs->field_count() != 0) {
+    ufs->DeleteByNumber(ufs->field(0).number());
+  }
+
+  ufs = Arena::Create<UnknownFieldSet>(&arena);
+  PopulateUFS(*ufs);
+  ufs->DeleteSubrange(0, ufs->field_count());
+}
+
 TEST_F(UnknownFieldSetTest, SerializeFastAndSlowAreEquivalent) {
   int size =
       WireFormat::ComputeUnknownFieldsSize(empty_message_.unknown_fields());
@@ -335,7 +441,6 @@ TEST_F(UnknownFieldSetTest, MergeFromMessageLite) {
   EXPECT_EQ(unknown_field.fixed32(), 42);
 }
 
-
 TEST_F(UnknownFieldSetTest, Clear) {
   // Clear the set.
   empty_message_.Clear();
@@ -518,13 +623,15 @@ TEST_F(UnknownFieldSetTest, UnknownEnumValue) {
 TEST_F(UnknownFieldSetTest, SpaceUsedExcludingSelf) {
   UnknownFieldSet empty;
   empty.AddVarint(1, 0);
-  EXPECT_EQ(sizeof(UnknownField), empty.SpaceUsedExcludingSelf());
+  RepeatedField<UnknownField> rep;
+  rep.Add();
+  EXPECT_EQ(rep.SpaceUsedExcludingSelf(), empty.SpaceUsedExcludingSelf());
 }
 
 TEST_F(UnknownFieldSetTest, SpaceUsed) {
   // Keep shadow vectors to avoid making assumptions about its capacity growth.
   // We imitate the push back calls here to determine the expected capacity.
-  std::vector<UnknownField> shadow_vector, shadow_vector_group;
+  RepeatedField<UnknownField> shadow_vector, shadow_vector_group;
   unittest::TestEmptyMessage empty_message;
 
   // Make sure an unknown field set has zero space used until a field is
@@ -534,8 +641,8 @@ TEST_F(UnknownFieldSetTest, SpaceUsed) {
   UnknownFieldSet* group = nullptr;
   const auto total = [&] {
     size_t result = base;
-    result += shadow_vector.capacity() * sizeof(UnknownField);
-    result += shadow_vector_group.capacity() * sizeof(UnknownField);
+    result += shadow_vector.SpaceUsedExcludingSelfLong();
+    result += shadow_vector_group.SpaceUsedExcludingSelfLong();
     if (str != nullptr) {
       result += sizeof(std::string);
       static const size_t sso_capacity = std::string().capacity();
@@ -552,30 +659,30 @@ TEST_F(UnknownFieldSetTest, SpaceUsed) {
 
   // Make sure each thing we add to the set increases the SpaceUsedLong().
   unknown_fields->AddVarint(1, 0);
-  shadow_vector.emplace_back();
+  shadow_vector.Add();
   EXPECT_EQ(total(), empty_message.SpaceUsedLong()) << "Var";
 
-  str = unknown_fields->AddLengthDelimited(1);
-  shadow_vector.emplace_back();
+  str =
+      internal::UnknownFieldSetTestPeer::AddLengthDelimited(*unknown_fields, 1);
+  shadow_vector.Add();
   EXPECT_EQ(total(), empty_message.SpaceUsedLong()) << "Str";
 
   str->assign(sizeof(std::string) + 1, 'x');
   EXPECT_EQ(total(), empty_message.SpaceUsedLong()) << "Str2";
 
   group = unknown_fields->AddGroup(1);
-  shadow_vector.emplace_back();
+  shadow_vector.Add();
   EXPECT_EQ(total(), empty_message.SpaceUsedLong()) << "Group";
 
   group->AddVarint(1, 0);
-  shadow_vector_group.emplace_back();
+  shadow_vector_group.Add();
   EXPECT_EQ(total(), empty_message.SpaceUsedLong()) << "Group2";
 
   unknown_fields->AddVarint(1, 0);
-  shadow_vector.emplace_back();
+  shadow_vector.Add();
   EXPECT_EQ(total(), empty_message.SpaceUsedLong()) << "Var2";
 }
 
-
 TEST_F(UnknownFieldSetTest, Empty) {
   UnknownFieldSet unknown_fields;
   EXPECT_TRUE(unknown_fields.empty());
@@ -664,7 +771,12 @@ TEST_F(UnknownFieldSetTest, SerializeToString) {
   field_set.AddVarint(1, -1);
   field_set.AddVarint(2, -2);
   field_set.AddLengthDelimited(44, "str");
-  field_set.AddLengthDelimited(44, "byv");
+  field_set.AddLengthDelimited(44, std::string("byv"));
+  field_set.AddLengthDelimited(44,
+                               absl::Cord("this came from cord and is long"));
+  *internal::UnknownFieldSetTestPeer::AddLengthDelimited(field_set, 44) =
+      "0123456789";
+
   field_set.AddFixed32(7, 7);
   field_set.AddFixed64(8, 8);
 
@@ -673,10 +785,8 @@ TEST_F(UnknownFieldSetTest, SerializeToString) {
   group_field_set = field_set.AddGroup(46);
   group_field_set->AddVarint(47, 2048);
 
-  unittest::TestAllTypes message;
-  std::string serialized_message;
-  ASSERT_TRUE(field_set.SerializeToString(&serialized_message));
-  ASSERT_TRUE(message.ParseFromString(serialized_message));
+  unittest::TestAllTypes message =
+      UnknownToProto<unittest::TestAllTypes>(field_set);
 
   EXPECT_EQ(message.optional_int32(), -1);
   EXPECT_EQ(message.optional_int64(), -2);
@@ -684,8 +794,9 @@ TEST_F(UnknownFieldSetTest, SerializeToString) {
   EXPECT_EQ(message.optional_uint64(), 4);
   EXPECT_EQ(message.optional_fixed32(), 7);
   EXPECT_EQ(message.optional_fixed64(), 8);
-  EXPECT_EQ(message.repeated_string(0), "str");
-  EXPECT_EQ(message.repeated_string(1), "byv");
+  EXPECT_THAT(message.repeated_string(),
+              ElementsAre("str", "byv", "this came from cord and is long",
+                          "0123456789"));
   EXPECT_EQ(message.repeatedgroup(0).a(), 1024);
   EXPECT_EQ(message.repeatedgroup(1).a(), 2048);
 }
@@ -726,12 +837,47 @@ TEST_F(UnknownFieldSetTest, SerializeToCord_TestPackedTypes) {
   ASSERT_TRUE(field_set.SerializeToCord(&cord));
 
   unittest::TestPackedTypes message;
-  ASSERT_TRUE(message.ParseFromCord(cord));
+  ASSERT_TRUE(message.ParseFromString(cord));
   EXPECT_THAT(message.packed_int32(), ElementsAre(-1, -2, -3, -4));
   EXPECT_THAT(message.packed_uint64(), ElementsAre(5, 6, 7));
 }
 
-}  // namespace
+TEST(UnknownFieldTest, SettersOverrideTheDataProperly) {
+  using T = unittest::TestAllTypes;
+  UnknownFieldSet set;
+  set.AddVarint(T::kOptionalInt32FieldNumber, 2);
+  set.AddFixed32(T::kOptionalFixed32FieldNumber, 3);
+  set.AddFixed64(T::kOptionalFixed64FieldNumber, 4);
+  set.AddLengthDelimited(T::kOptionalStringFieldNumber, "5");
 
+  T message = UnknownToProto<T>(set);
+
+  EXPECT_EQ(message.optional_int32(), 2);
+  EXPECT_EQ(message.optional_fixed32(), 3);
+  EXPECT_EQ(message.optional_fixed64(), 4);
+  EXPECT_EQ(message.optional_string(), "5");
+
+  set.mutable_field(0)->set_varint(22);
+  set.mutable_field(1)->set_fixed32(33);
+  set.mutable_field(2)->set_fixed64(44);
+  set.mutable_field(3)->set_length_delimited("55");
+
+  message = UnknownToProto<T>(set);
+
+  EXPECT_EQ(message.optional_int32(), 22);
+  EXPECT_EQ(message.optional_fixed32(), 33);
+  EXPECT_EQ(message.optional_fixed64(), 44);
+  EXPECT_EQ(message.optional_string(), "55");
+
+  set.mutable_field(3)->set_length_delimited(std::string("555"));
+  message = UnknownToProto<T>(set);
+  EXPECT_EQ(message.optional_string(), "555");
+
+  set.mutable_field(3)->set_length_delimited(absl::Cord("5555"));
+  message = UnknownToProto<T>(set);
+  EXPECT_EQ(message.optional_string(), "5555");
+}
+
+}  // namespace
 }  // namespace protobuf
 }  // namespace google
