@@ -20,12 +20,10 @@
 #include <utility>
 
 #include "google/protobuf/descriptor.pb.h"
-#include "absl/container/fixed_array.h"
 #include "absl/container/flat_hash_map.h"
 #include "absl/log/absl_check.h"
 #include "absl/log/absl_log.h"
 #include "absl/strings/escaping.h"
-#include "absl/strings/match.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/str_format.h"
 #include "google/protobuf/descriptor.h"
@@ -529,6 +527,15 @@ bool MessageDifferencer::FieldBefore(const FieldDescriptor* field1,
 
 bool MessageDifferencer::Compare(const Message& message1,
                                  const Message& message2) {
+  const Descriptor* descriptor1 = message1.GetDescriptor();
+  const Descriptor* descriptor2 = message2.GetDescriptor();
+  if (descriptor1 != descriptor2) {
+    ABSL_DLOG(FATAL) << "Comparison between two messages with different "
+                     << "descriptors. " << descriptor1->full_name() << " vs "
+                     << descriptor2->full_name();
+    return false;
+  }
+
   std::vector<SpecificField> parent_fields;
   force_compare_no_presence_fields_.clear();
   force_compare_failure_triggering_fields_.clear();
@@ -602,17 +609,8 @@ bool MessageDifferencer::CompareWithFields(
 bool MessageDifferencer::Compare(const Message& message1,
                                  const Message& message2, int unpacked_any,
                                  std::vector<SpecificField>* parent_fields) {
-  const Descriptor* descriptor1 = message1.GetDescriptor();
-  const Descriptor* descriptor2 = message2.GetDescriptor();
-  if (descriptor1 != descriptor2) {
-    ABSL_DLOG(FATAL) << "Comparison between two messages with different "
-                     << "descriptors. " << descriptor1->full_name() << " vs "
-                     << descriptor2->full_name();
-    return false;
-  }
-
   // Expand google.protobuf.Any payload if possible.
-  if (descriptor1->full_name() == internal::kAnyFullTypeName) {
+  if (message1.GetDescriptor()->full_name() == internal::kAnyFullTypeName) {
     std::unique_ptr<Message> data1;
     std::unique_ptr<Message> data2;
     if (unpack_any_field_.UnpackAny(message1, &data1) &&
@@ -823,6 +821,12 @@ static PROTOBUF_NOINLINE MessageDifferencer::SpecificField& PushSpecificField(
   return fields->back();
 }
 
+void MessageDifferencer::ForceCompareField(const FieldDescriptor* field) {
+  if (force_compare_no_presence_fields_.contains(field)) {
+    force_compare_failure_triggering_fields_.emplace(field->full_name());
+  }
+}
+
 bool MessageDifferencer::CompareWithFieldsInternal(
     const Message& message1, const Message& message2, int unpacked_any,
     const std::vector<const FieldDescriptor*>& message1_fields,
@@ -897,8 +901,8 @@ bool MessageDifferencer::CompareWithFieldsInternal(
     } else if (FieldBefore(field2, field1)) {
       const bool ignore_field =
           IsIgnored(message1, message2, field2, *parent_fields);
-      if (!ignore_field && force_compare_no_presence_fields_.contains(field2)) {
-        force_compare_failure_triggering_fields_.insert(field2->full_name());
+      if (!ignore_field) {
+        ForceCompareField(field2);
       }
 
       // Field 2 is not in the field list for message 1.
@@ -989,9 +993,7 @@ bool MessageDifferencer::CompareWithFieldsInternal(
       fieldDifferent = !CompareFieldValueUsingParentFields(
           message1, message2, unpacked_any, field1, -1, -1, parent_fields);
 
-      if (force_compare_no_presence_fields_.contains(field1)) {
-        force_compare_failure_triggering_fields_.insert(field1->full_name());
-      }
+      ForceCompareField(field1);
 
       if (reporter_ != nullptr) {
         SpecificField& specific_field = PushSpecificField(parent_fields);
@@ -2111,7 +2113,7 @@ MessageDifferencer::StreamReporter::~StreamReporter() {
 void MessageDifferencer::StreamReporter::PrintPath(
     const std::vector<SpecificField>& field_path, bool left_side) {
   for (size_t i = 0; i < field_path.size(); ++i) {
-    SpecificField specific_field = field_path[i];
+    const SpecificField& specific_field = field_path[i];
 
     if (specific_field.field != nullptr &&
         specific_field.field->name() == "value") {
@@ -2382,3 +2384,5 @@ MessageDifferencer::CreateMultipleFieldsMapKeyComparator(
 }  // namespace util
 }  // namespace protobuf
 }  // namespace google
+
+#include "google/protobuf/port_undef.inc"
